/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry;

public final class RegistryConstants {
    /**
     * These constants are used to specify the core registry to use in the registry server. This is
     * specified as a init parameter of the Registry servlet.
     */
    public static final String REGISTRY_TYPE_PARAMETER = "registryType";
    public static final String JDBC_REGISTRY_TYPE = "JDBCRegistry";
    public static final String REMOTE_REGISTRY_TYPE = "RemoteRegistry";
    public static final String INMEM_REGISTRY_TYPE = "InMemoryRegistry";
    public static final String STATUS_MESSAGE_NAME = "edit_status";

    public static final String REGISTRY_CONFIG_PATH = "registry.config.path";
    public static final String REGISTRY_CONTEXT = "registry.context";

    /** Built in user names */
    public static final String SYSTEM_USER = "system";
    public static final String SYSTEM_PASSWORD = "system";
    public static final String ANONYMOUS_USER = "anonymous";
    public static final String ANONYMOUS_PASSWORD = "guest";
    public static final String GUESTS_ROLE = "guests";
    public static final String EVERYONE_ROLE = "everyone";
    public static final String ADMIN_USER = "admin";
    public static final String ADMIN_PASSWORD = "admin";
    public static final String ADMIN_ROLE = "admin";

    /** Known path parameter names */
    public static final String VERSION_PARAMETER_NAME = "version";

    public static final String ROOT_PATH = "/";
    public static final String QUERIES_COLLECTION_NAME = "queries";
    public static final String USERS_COLLECTION_NAME = "users";
    public static final String PATH_SEPARATOR = "/";

    public static final String TAG_PROPERTY = "tag";

    public static final String REGISTRY = "CoreRegistry";
    public static final String SYSTEM_REGISTRY = "SystemRegistry";
    public static final String USER_REGISTRY = "user_registry";
    public static final String REGISTRY_REALM = "registry_realm";


    public static final String SESSION_PROPERTY = "SessionObject";
    public static final String TAG_REGISTRY = "/view";
    public static final String TAG_UPLOAD = "/upload";
    public static final String ACTION = "action";

    public static final String CONTENT_UI_MODE = "registry.ui.mode";
    public static final String BROWSE_MODE = "browse.mode";
    public static final String EDIT_MODE = "edit.mode";
    public static final String NEW_MODE = "new.mode";

    // rendering options in the UI.
    public static final String UI_RENDERING_METHOD_PROPERTY = "ui.rendering.method";
    public static final String R_RAW = "raw";
    public static final String R_VIEW_TEXT = "view.text";
    public static final String R_EDIT_TEXT = "edit.text";
    public static final String R_NEW_TEXT = "new.text";
    public static final String R_VIEW_XML = "view.xml";
    public static final String R_EDIT_XML = "edit.xml";
    public static final String R_NEW_XML = "new.xml";
    public static final String R_GENERAL = "general";

    // built-in view/edit names
    public static final String DEFAULT_VIEW_NAME = "default";
    public static final String RAW_VIEW_NAME = "raw";
    public static final String TEXT_VIEW_NAME = "text";
    public static final String XML_VIEW_NAME = "xml";
    public static final String UNDEFINED_VIEW_NAME = "undefined";

    // Directory listing
    public static final String DIRECTORY_ELEMENT = "directory";
    public static final String FILE_ELEMENT = "file";
    public static final String ATT_NAME = "name";
    public static final String ATT_PATH = "path";

    public static final String REGISTRY_INSTANCE = "registry";
    public static final String PATH = "path";
    public static final int ACTIVE_STATE = 100;
    public static final int DELETED_STATE = 101;
    public static final int ITEMPS_PER_PAGE = 10;

    // dependency types for setting dependencies between resources
    public static final String DEFAULT_DEPENDENCY = "default";

    // internal media types
    public static final String DEFAULT_MEDIA_TYPE = "default";
    public static final String SQL_QUERY_MEDIA_TYPE = "application/vnd.sql.query";
    public static final String COMMENT_MEDIA_TYPE = "comment";
    public static final String RATING_MEDIA_TYPE = "rating";
    public static final String TAG_MEDIA_TYPE = "tag";

    // built in media types

    public static final String WSDL_MEDIA_TYPE = "application/wsdl+xml";
    public static final String XSD_MEDIA_TYPE = "application/x-xsd+xml";
    public static final String SYNPASE_REPOSITORY_MEDIA_TYPE = "synapse-repo";
    public static final String SYNAPSE_CONF_COLLECTION_MEDIA_TYPE = "synapse-conf";
    public static final String SYNAPSE_SEQUENCE_COLLECTION_MEDIA_TYPE = "synapse-sequences";
    public static final String SYNAPSE_ENDPOINT_COLLECTION_MEDIA_TYPE = "synapse-endpoints";
    public static final String SYNAPSE_PROXY_SERVICES_COLLECTION_MEDIA_TYPE =
            "synapse-proxy-services";
    public static final String SYNAPSE_TASKS_COLLECTION_MEDIA_TYPE = "synapse-tasks";
    public static final String SYNAPSE_ENTRIES_COLLECTION_MEDIA_TYPE = "synapse-entries";

    public static final String SYNAPSE_CONF_COLLECTION_NAME = "conf";
    public static final String SYNAPSE_SEQUENCES_COLLECTION_NAME = "sequences";
    public static final String SYNAPSE_ENDPOINT_COLLECTION_NAME = "endpoints";
    public static final String SYNAPSE_PROXY_SERVICES_COLLECTION_NAME = "proxy-services";
    public static final String SYNAPSE_TASKS_COLLECTION_NAME = "tasks";
    public static final String SYNAPSE_ENTRIES_COLLECTION_NAME = "entries";

    public static final String AXIS2_CONF_COLLECTION_MEDIA_TYPE = "axis2-conf";
    public static final String AXIS2_SERVICES_COLLECTION_MEDIA_TYPE = "axis2-services";
    public static final String AXIS2_MODULES_COLLECTION_MEDIA_TYPE = "axis2-modules";

    public static final String AXIS2_CONF_COLLECTION_NAME = "conf";
    public static final String AXIS2_SERVICES_COLLECTION_NAME = "services";
    public static final String AXIS2_MODULES_COLLECTION_NAME = "modules";

    // result types of dynamic queries
    public static final String RESULT_TYPE_PROPERTY_NAME = "resultType";
    public static final String RESOURCES_RESULT_TYPE = "Resource";
    public static final String COMMENTS_RESULT_TYPE = "Comments";
    public static final String RATINGS_RESULT_TYPE = "Ratings";
    public static final String TAGS_RESULT_TYPE = "Tags";

    public static final String RESOURCES_JSP = "/admin/resources.jsp";
    public static final String RESOURCE_DETAILS_JSP = "/admin/resources_details.jsp";
    public static final String ERROR_MESSAGE = "error.message";
    public static final String ERROR_JSP = "/admin/error.jsp";
    public static final String PATH_ATTR = "path";

    // Separator used to access Registry metadata - i.e. "/resource$tags"
    public static final String URL_SEPARATOR = ";";
    public static final String URL_PARAMETER_SEPARATOR = ":";

    public static final String VIEW_ACTION = URL_SEPARATOR + "view";
    public static final String BROWSE_PROPERTY = "view";

    public static final String EDIT_ACTION = URL_SEPARATOR + "edit";
    public static final String EDIT_PROPERTY = "edit";

    public static final String NEW_PROPERTY = "new";

    public static final String UI_CONTENT_PROPERTY = "UI.content";
    public static final String UI_HTML = "UI.HTML";
    public static final String UI_TEXT = "UI.text";
    public static final String UI_XML = "UI.XML";
    public static final String UI_NONE = "UI.none";

    public static final String TEXT_INPUT_NAME = "generic-text-input";
    public static final String TEXT_EDIT_PROCESSOR_KEY = "system.text.edit.processor";

    public static final String CUSTOM_EDIT_PROCESSOR_KEY = "edit-processor";
    public static final String VIEW_KEY = "view-key";
    public static final String VIEW_TYPE = "view-type";
    public static final String EDIT_VIEW_TYPE = "edit";
    public static final String NEW_VIEW_TYPE = "new";

    public static final String RESOURCES_PATH = "resources";

    public static final String REGISTRY_NAMESPACE = "http://wso2.org/registry";

    private RegistryConstants() {
    }
}
