/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.jdbc.dao.ResourceVersionDAO;
import org.wso2.registry.jdbc.utils.Transaction;
import org.wso2.registry.jdbc.filecache.FileManager;
import org.wso2.registry.exceptions.RegistryException;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.SQLException;

public class ResourceVersionImpl extends ResourceImpl {

    private static Log log = LogFactory.getLog(ResourceVersionImpl.class);

    ResourceVersionDAO resourceVersionDAO = new ResourceVersionDAO();

    public InputStream getContentStream() throws RegistryException {

        if (fileBasedContentID != null) {
            return FileManager.getInstance().getFileBasedInputStream(fileBasedContentID);
        }

        if (dbBasedContentID != null) {

            if (Transaction.isStarted()) {

                Connection conn = Transaction.getConnection();
                resourceVersionDAO.createFileBasedContentFromDB(dbBasedContentID, conn);

            } else if (dataSource != null) {

                Connection conn = null;

                try {
                    conn = dataSource.getConnection();
                    resourceVersionDAO.createFileBasedContentFromDB(dbBasedContentID, conn);

                } catch (Exception e) {
                    String msg =
                            "Failed to get the input stream for the content of resource: " + path;
                    log.error(msg, e);
                    throw new RegistryException(msg, e);

                } finally {
                    if (conn != null) {
                        try {
                            conn.close();
                        } catch (SQLException e) {
                            log.error("Failed to close the database connection.", e);
                        }
                    }
                }
            }

            fileBasedContentID = dbBasedContentID;
            return FileManager.getInstance().getFileBasedInputStream(fileBasedContentID);
        }

        if (content != null) {

            if (content instanceof byte[]) {
                return new BufferedInputStream(new ByteArrayInputStream((byte[]) content));

            } else if (content instanceof String) {
                byte[] contentBytes = ((String) content).getBytes();
                return new BufferedInputStream(new ByteArrayInputStream(contentBytes));
            }
        }

        return null;
    }
}
