/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.ResourceImpl;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

public class RemoteResourceImpl extends ResourceImpl {
    URL contentURL;
    String authorizationString;
    InputStream contentStream;

    public void setContentURL(URL contentURL) {
        this.contentURL = contentURL;
    }

    public String getAuthorizationString() {
        return authorizationString;
    }

    public void setAuthorizationString(String authorizationString) {
        this.authorizationString = authorizationString;
    }

    public InputStream getContentStream() throws RegistryException {
        InputStream is = super.getContentStream();
        if (is != null) return is;

        if (contentURL != null) {
            try {
                URLConnection uc = contentURL.openConnection();
                if (authorizationString != null) {
                    uc.setRequestProperty("Authorization", authorizationString);
                }
                contentStream = uc.getInputStream();
            } catch (IOException e) {
                throw new RegistryException("Couldn't open stream to source URL " + contentURL);
            }
        }

        return contentStream;
    }
}
