/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.handlers.builtin;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.jdbc.handlers.Handler;
import org.wso2.registry.jdbc.handlers.RequestContext;

public class SQLQueryHandler extends Handler {

    public Resource get(RequestContext requestContext) throws RegistryException {

        Resource resource = requestContext.getResource();
        if (resource == null) {
            String queryPath = requestContext.getResourcePath().getPath();
            resource = requestContext.getRepository().get(queryPath);
            requestContext.setResource(resource);
        }

        Object content = resource.getContent();
        if (content instanceof byte[]) {
            resource.setContent(new String((byte[])content));
        }

        requestContext.setProcessingComplete(true);
        return resource;
    }

    public void put(RequestContext requestContext) throws RegistryException {

        Resource resource = requestContext.getResource();

        Object content = resource.getContent();
        if (content instanceof String) {
            String textContent = (String)content;
            resource.setContent(textContent.getBytes());
        }

        String queryPath = requestContext.getResourcePath().getPath();
        requestContext.getRepository().put(queryPath, resource);
        requestContext.setProcessingComplete(true);
    }

    public void importResource(RequestContext requestContext) throws RegistryException {
    }

    public void delete(RequestContext requestContext) throws RegistryException {
    }

    public void putChild(RequestContext requestContext) throws RegistryException {
    }

    public void importChild(RequestContext requestContext) throws RegistryException {
    }
}
