/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.realm;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.users.UserStoreException;

public class RegistryAccessControlAdmin implements AccessControlAdmin {

    private AccessControlAdmin coreACA;

    public RegistryAccessControlAdmin(AccessControlAdmin accessControlAdmin) {
        coreACA = accessControlAdmin;
    }

    //public RegistryAccessControlAdmin(DataSource dataSource, String algo) {
    //    super(dataSource, algo);
    //}

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        coreACA.clearUserAuthorization(userName, resourceId, action);
    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER)) {

            String msg = "Could not change authorizations of the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        coreACA.denyUser(userName, resourceId, action);
    }


    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        coreACA.clearRoleAuthorization(roleName, resourceId, action);
    }


    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE)) {

            String msg = "Could not change authorizations of the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        coreACA.denyRole(roleName, resourceId, action);
    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        coreACA.authorizeUser(userName, resourceId, action);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        coreACA.authorizeRole(roleName, resourceId, action);
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        coreACA.clearResourceAuthorizations(resourceId);
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
        coreACA.copyAuthorizations(fromResourceId, toResourceId);
    }

    public boolean isUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        return coreACA.isUserAuthorized(userName, resourceId, action);
    }

    public boolean isRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {
        return coreACA.isRoleAuthorized(roleName, resourceId, action);
    }

    public String[] getAllowedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getAllowedUsersForResource(resourceId, action);
    }

    public String[] getDeniedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getDeniedUsersForResource(resourceId, action);
    }

    public String[] getAllowedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getAllowedRolesForResource(resourceId, action);
    }

    public String[] getDeniedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        return coreACA.getDeniedRolesForResource(resourceId, action);
    }
}
