/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.utils;

import org.apache.commons.dbcp.BasicDataSource;

import javax.sql.DataSource;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;

/**
 * Datasource for all registry data access operations. All the classes executing inside the
 * JDBCRegistry should get connections form this class and such classes should not deal with
 * any transaction related methods (i.e. should not call conn.setAutoCommit(), conn.rollbackTransaction() or
 * conn.commit(). Such transactional issues are handled only in the JDBCRegistry.
 */
public class RegistryDataSource implements DataSource {

    private DataSource dataSource = null;

    public RegistryDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public RegistryDataSource(String dbURL, String driverName, String userName, String password) {

        BasicDataSource basicDataSource = new BasicDataSource();
        basicDataSource.setUrl(dbURL);
        basicDataSource.setDriverClassName(driverName);
        basicDataSource.setUsername(userName);
        basicDataSource.setPassword(password);

        this.dataSource = basicDataSource;
    }

    public Connection getConnection() throws SQLException {
        return dataSource.getConnection();
    }

    public Connection getConnection(String username, String password) throws SQLException {
        return dataSource.getConnection(username, password);
    }

    public PrintWriter getLogWriter() throws SQLException {
        return dataSource.getLogWriter();
    }

    public void setLogWriter(PrintWriter out) throws SQLException {
        dataSource.setLogWriter(out);
    }

    public void setLoginTimeout(int seconds) throws SQLException {
        dataSource.setLoginTimeout(seconds);
    }

    public int getLoginTimeout() throws SQLException {
        return dataSource.getLoginTimeout();
    }

    public <T> T unwrap(Class<T> iface) throws SQLException {
        return null;
    }

    public boolean isWrapperFor(Class<?> iface) throws SQLException {
        return false;
    }
}
