package org.wso2.registry.users;

import org.wso2.authenticator.Authenticator;

/**
 *  Registry access users through this interface
 */
public interface UserRealm {
    
    /**
     * Returns the realm configuration bean.
     * 
     * @throws UserManagerException
     */
    public Object getRealmConfiguration() throws UserStoreException;

    /**
     * Initializes the realm using the properties from the configuration bean
     * passed in.
     * 
     * @throws UserManagerException
     */
    public void init(Object configBean) throws UserStoreException;

    /**
     * The returned object can perform authentication checks
     * 
     * @throws UserManagerException
     */
    public Authenticator getAuthenticator() throws UserStoreException;

    /**
     * The returned object can perform authorization checks
     * 
     * @throws UserManagerException
     */
    public Authorizer getAuthorizer() throws UserStoreException;

    /**
     * The returned object can read data from user store
     */
    public UserStoreReader getUserStoreReader() throws UserStoreException;

    /**
     * The returned object can manipulate the user store
     * 
     * @throws UserManagerException
     */
    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException;

    /**
     * The returned object can manipulate access controls
     * 
     * @throws UserManagerException
     */
    public AccessControlAdmin getAccessControlAdmin()
            throws UserStoreException;

}
