/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.users.def;

import org.apache.axiom.om.util.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.authenticator.Authenticator;
import org.wso2.authenticator.AuthenticatorException;
import org.wso2.registry.i18n.Messages;

import javax.sql.DataSource;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

/**
 * @see org.wso2.usermanager.Authenticator
 */
public class DefaultAuthenticator implements Authenticator {

    private static Log log = LogFactory.getLog(DefaultAuthenticator.class);

    protected DataSource dataSource = null;

    protected DefaultRealm realm = null;

    public DefaultAuthenticator(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    /**
     * @see org.wso2.usermanager.Authenticator#authenticate(String, Object)
     */
    public boolean authenticate(String userName, Object credentials) throws AuthenticatorException {
        boolean isAuth = false;
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new AuthenticatorException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getUserStmt = dbConnection
                    .prepareStatement(DefaultRealmConstants.GET_USER_SQL);
            getUserStmt.setString(1, userName);
            ResultSet rs = getUserStmt.executeQuery();
            if (rs.next()) {
                if (credentials != null) {
                    MessageDigest dgst = MessageDigest.getInstance("MD5");
                    dgst.update(((String) credentials).getBytes());
                    String dbCred = rs.getString(DefaultRealmConstants.COLUMN_NAME_CREDENTIAL);
                    isAuth = Base64.encode(dgst.digest()).equals(dbCred);
                }
            }
            getUserStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new AuthenticatorException(Messages.getMessage("errorReadingFromUserStore"), e);
        } catch (NoSuchAlgorithmException e) {
            log.debug(e);
            throw new AuthenticatorException(Messages.getMessage("errorCreatingPasswordDigest"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new AuthenticatorException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return isAuth;
    }

}
