/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.users.def.util;

import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.def.DefaultRealmConstants;

import java.sql.*;
import java.util.LinkedList;
import java.util.List;

public class DefaultDatabaseUtil {
    // TODO Start -delete
    public static String userTable = "CREATE TABLE UM_USERS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "USER_NAME VARCHAR(255) NOT NULL,"
            + "USER_PASSWORD VARCHAR(255) NOT NULL, " + "PRIMARY KEY (ID), " + "UNIQUE(USER_NAME))";

    public static String userAttributeTable = "CREATE TABLE UM_USER_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY," + "ATTR_NAME VARCHAR(255) NOT NULL,"
            + "ATTR_VALUE VARCHAR(255), " + "USER_ID INTEGER, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String roleTable = "CREATE TABLE UM_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_NAME VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_NAME))";

    public static String roleAttributeTable = "CREATE TABLE UM_ROLE_ATTRIBUTES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ATTR_NAME VARCHAR(255) NOT NULL, "
            + "ATTR_VALUE VARCHAR(255), " + "ROLE_ID INTEGER, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String permissionTable = "CREATE TABLE UM_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, "
            + "RESOURCE_ID VARCHAR(255) NOT NULL, " + "ACTION VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID))";

    public static String rolePermissionTable = "CREATE TABLE UM_ROLE_PERMISSIONS ( "
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "ROLE_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, ROLE_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String userPermissionTable = "CREATE TABLE UM_USER_PERMISSIONS ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "PERMISSION_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, USER_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String userRolesTable = "CREATE TABLE UM_USER_ROLES ("
            + "ID INTEGER GENERATED BY DEFAULT AS IDENTITY, " + "ROLE_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "UNIQUE (USER_ID, ROLE_ID), "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static void createDatabase(Connection conn) throws SQLException, UserStoreException {

        Statement s = conn.createStatement();
        s.executeUpdate(userTable);
        s.executeUpdate(roleTable);
        s.executeUpdate(userAttributeTable);
        s.executeUpdate(roleAttributeTable);
        s.executeUpdate(permissionTable);
        s.executeUpdate(rolePermissionTable);
        s.executeUpdate(userPermissionTable);
        s.executeUpdate(userRolesTable);
        conn.commit();

    }

    // TODO End -delete

    public static int getRoleId(Connection dbConnection, String roleName) throws UserStoreException {
        int id = -1;
        try {
            PreparedStatement getRolIdStmt = dbConnection
                    .prepareStatement(DefaultRealmConstants.GET_ROLE_ID_SQL);
            getRolIdStmt.setString(1, roleName);
            ResultSet rs = getRolIdStmt.executeQuery();
            if (rs.next()) {
                id = rs.getInt(DefaultRealmConstants.COLUMN_NAME_ID);
            }

            getRolIdStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return id;
    }

    public static int getUserId(Connection dbConnection, String userName) throws UserStoreException {
        int id = -1;
        try {
            PreparedStatement getUserIdStmt = dbConnection
                    .prepareStatement(DefaultRealmConstants.GET_USER_ID_SQL);
            getUserIdStmt.setString(1, userName);
            ResultSet rs = getUserIdStmt.executeQuery();
            if (rs.next()) {
                id = rs.getInt(DefaultRealmConstants.COLUMN_NAME_ID);
            }
            getUserIdStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return id;
    }

    public static String[] getUserRoles(Connection dbConnection, String userName)
            throws UserStoreException {
        String[] names = new String[0];
        try {
            PreparedStatement getUserRolesStmt = dbConnection
                    .prepareStatement(DefaultRealmConstants.GET_USER_ROLES_SQL);
            getUserRolesStmt.setString(1, userName);
            ResultSet rs = getUserRolesStmt.executeQuery();
            List lst = new LinkedList();
            String colName = DefaultRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getUserRolesStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return names;
    }

}
