package org.wso2.registry.users.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.def.DefaultRealmConfig;

public class HybridAuthorizer implements Authorizer {

    /** DBCP datasource */
    protected DataSource dataSource = null;

    /** logger */
    private static Log log = LogFactory.getLog(HybridAuthorizer.class);

    /** Algorithm to authorize users */
    private String algorithm = null;

    public HybridAuthorizer(DataSource dataSource, String algo) {
        this.dataSource = dataSource;
        this.algorithm = algo;
    }

    public boolean isRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {

        Boolean isAuthorized = getRoleAuthorized(roleName, resourceId, action);
        return isAuthorized != null && isAuthorized;
    }

    public boolean isUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        Boolean isAuthorized = null;

        if (algorithm.equals(DefaultRealmConfig.PERMISSION_USER_ONLY)) {
            isAuthorized = getUserAuthorized(userName, resourceId, action);
        } else if (algorithm.equals(DefaultRealmConfig.PERMISSION_BLOCK_FIRST)) {
            isAuthorized = getUserAuthorizationConsideringRoles(userName, resourceId, action);
        }

        return isAuthorized != null && isAuthorized;
    }

    public String[] getAllowedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getAuthorizedUsersForResourceStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ALLOWED_USERS_ON_RESOURCE_SQL);
            getAuthorizedUsersForResourceStmt.setString(1, resourceId);
            getAuthorizedUsersForResourceStmt.setString(2, action);
            ResultSet rs = getAuthorizedUsersForResourceStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_USER_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getAuthorizedUsersForResourceStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public String[] getDeniedUsersForResource(String resourceId, String action)
            throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getAuthorizedUsersForResourceStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_DENIED_USERS_ON_RESOURCE_SQL);
            getAuthorizedUsersForResourceStmt.setString(1, resourceId);
            getAuthorizedUsersForResourceStmt.setString(2, action);
            ResultSet rs = getAuthorizedUsersForResourceStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_USER_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getAuthorizedUsersForResourceStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public String[] getAllowedRolesForResource(String resourceId, String action)
            throws UserStoreException {
        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getAuthorizedRolesForResourceStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ALLOWED_ROLES_FOR_RESOURCE_SQL);
            getAuthorizedRolesForResourceStmt.setString(1, resourceId);
            getAuthorizedRolesForResourceStmt.setString(2, action);
            ResultSet rs = getAuthorizedRolesForResourceStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getAuthorizedRolesForResourceStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;
    }

    public String[] getDeniedRolesForResource(String resourceId, String action)
            throws UserStoreException {

        String[] names = new String[0];
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getAuthorizedRolesForResourceStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_DENIED_ROLES_FOR_RESOURCE_SQL);
            getAuthorizedRolesForResourceStmt.setString(1, resourceId);
            getAuthorizedRolesForResourceStmt.setString(2, action);
            ResultSet rs = getAuthorizedRolesForResourceStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getAuthorizedRolesForResourceStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return names;

    }

    private Boolean getUserAuthorizationConsideringRoles(String userName, String resourceId,
            String action) throws UserStoreException {
        Boolean isAuthorized = getUserAuthorized(userName, resourceId, action);

        boolean onceAllowed = false;
        if (isAuthorized == null) {
            String roles[] = new String[0];
            Connection dbConnection = null;
            try {
                dbConnection = dataSource.getConnection();
                if (dbConnection == null) {
                    throw new UserStoreException(Messages.getMessage("null_connection"));
                }
                dbConnection.setAutoCommit(false);
                roles = getUserRoles(dbConnection, userName);
            } catch (SQLException e) {
                log.debug(e);
                throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
            } finally {
                try {
                    if (dbConnection != null) {
                        dbConnection.close();
                    }
                } catch (SQLException e) {
                    throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
                }
            }

            for (String roleName : roles) {
                Boolean roleAuth = getRoleAuthorized(roleName, resourceId, action);
                if (roleAuth != null) {
                    if (!roleAuth) {
                        isAuthorized = roleAuth;
                        break;
                    } else {
                        onceAllowed = true;
                    }
                }
            }
        }

        if (isAuthorized == null && onceAllowed) {
            isAuthorized = true;
        }

        return isAuthorized;
    }

    private Boolean getRoleAuthorized(String roleName, String resourceId, String action)
            throws UserStoreException {
        Boolean isAuthorized = null;
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement isRoleAuthorizedStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_ROLE_AUTHORIZED_SQL);
            isRoleAuthorizedStmt.setString(1, resourceId);
            isRoleAuthorizedStmt.setString(2, action);
            isRoleAuthorizedStmt.setString(3, roleName);
            ResultSet rs = isRoleAuthorizedStmt.executeQuery();
            if (rs.next()) {
                isAuthorized = rs.getBoolean(HybridRealmConstants.COLUMN_NAME_IS_ALLOWED);
            }
            isRoleAuthorizedStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return isAuthorized;
    }

    private Boolean getUserAuthorized(String userName, String resourceId, String action)
            throws UserStoreException {
        Boolean result = null;
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement isUserAuthorizedStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_USER_AUTHORIZED_SQL);
            isUserAuthorizedStmt.setString(1, resourceId);
            isUserAuthorizedStmt.setString(2, action);
            isUserAuthorizedStmt.setString(3, userName);
            ResultSet rs = isUserAuthorizedStmt.executeQuery();
            if (rs.next()) {
                result = rs.getBoolean(HybridRealmConstants.COLUMN_NAME_IS_ALLOWED);
            }
            isUserAuthorizedStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
        return result;
    }

    protected static String[] getUserRoles(Connection dbConnection, String userName)
            throws UserStoreException {
        String[] names = new String[0];
        try {
            PreparedStatement getUserRolesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.GET_USER_ROLES_SQL);
            getUserRolesStmt.setString(1, userName);
            ResultSet rs = getUserRolesStmt.executeQuery();
            List lst = new LinkedList();
            String colName = HybridRealmConstants.COLUMN_NAME_ROLE_NAME;
            while (rs.next()) {
                lst.add(rs.getString(colName));
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
            getUserRolesStmt.close();
        } catch (SQLException e) {
            throw new UserStoreException("errorReadingFromUserStore", e);
        }
        return names;
    }

}
