/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.users.hybrid;

/**
 * DefaultRealm Constants
 *
 */
public class HybridRealmConstants {

    public static final String ADD_ROLE_SQL = "INSERT INTO UM_ROLES (ROLE_NAME) VALUES (?)";
    public static final String ADD_USER_ROLE_SQL = "INSERT INTO UM_USER_ROLES (USER_ID, ROLE_ID) VALUES (?, ?)";
 
    public static final String ADD_ROLE_ATTRIBUTE_SQL = "INSERT INTO UM_ROLE_ATTRIBUTES (ATTR_NAME, ATTR_VALUE, ROLE_ID) VALUES (?, ?, ?)";
    public static final String ADD_PERMISSION_SQL = "INSERT INTO UM_PERMISSIONS (RESOURCE_ID, ACTION) VALUES (?, ?)";
    public static final String ADD_ROLE_PERMISSION_SQL = "INSERT INTO UM_ROLE_PERMISSIONS (PERMISSION_ID, IS_ALLOWED, ROLE_ID) VALUES (?, ?, ?)";
    public static final String ADD_USER_PERMISSION_SQL = "INSERT INTO UM_USER_PERMISSIONS (PERMISSION_ID, IS_ALLOWED, USER_ID) VALUES (?, ?, ?)";
 
    public static final String DELETE_ROLE_SQL = "DELETE FROM UM_ROLES WHERE ROLE_NAME = ?";

    public static final String DELETE_USER_ROLE_SQL = "DELETE FROM UM_USER_ROLES"
            + " WHERE USER_ID=?"
            + " AND ROLE_ID=(SELECT ID FROM UM_ROLES WHERE ROLE_NAME=?)";

    public static final String DELETE_ROLE_ATTRIBUTE_SQL = "DELETE FROM UM_ROLE_ATTRIBUTES"
            + " WHERE ROLE_ID = ?";

    //public static final String DELETE_USER_ATTRIBUTE_SQL = "DELETE FROM UM_USER_ATTRIBUTES"
      //      + " WHERE USER_ID = ?";

    public static final String DELETE_PERMISSION_ON_RESOURCE_SQL = "DELETE FROM UM_PERMISSIONS"
            + " WHERE RESOURCE_ID = ?";

    //public static final String GET_USER_SQL = "SELECT USER_NAME, USER_PASSWORD FROM UM_USERS WHERE USER_NAME=?";
    public static final String GET_ROLE_SQL = "SELECT * FROM UM_ROLES WHERE ROLE_NAME=?";

    public static final String DELETE_USER_PERMISSION_SQL = "DELETE FROM UM_USER_PERMISSIONS"
            + " WHERE USER_ID = ? AND PERMISSION_ID = ?";

    public static final String DELETE_ROLE_PERMISSION_SQL = "DELETE FROM UM_ROLE_PERMISSIONS"
            + " WHERE ROLE_ID = ? AND PERMISSION_ID = ?";

    public static final String GET_USER_ROLES_SQL = "SELECT UM_ROLES.ROLE_NAME"
        + " FROM UM_USER_ROLES, UM_ROLES WHERE UM_USER_ROLES.USER_ID=?"
        + " AND UM_ROLES.ID=UM_USER_ROLES.ROLE_ID";

    public static final String GET_ROLE_ATTRIBUTES_SQL = "SELECT *"
            + " FROM UM_ROLE_ATTRIBUTES, UM_ROLES"
            + " WHERE UM_ROLES.ID = UM_ROLE_ATTRIBUTES.ROLE_ID"
            + " AND UM_ROLES.ROLE_NAME=?";

/*    public static final String GET_USER_ATTRIBUTES_SQL = "SELECT *"
            + " FROM UM_USER_ATTRIBUTES, UM_USERS"
            + " WHERE UM_USERS.ID = UM_USER_ATTRIBUTES.USER_ID"
            + " AND UM_USERS.USER_NAME=?";*/

    public static final String GET_PERMISSION_SQL = "SELECT ID"
            + " FROM UM_PERMISSIONS WHERE UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?";

    public static final String GET_ROLE_AUTHORIZED_SQL = "SELECT"
            + " UM_ROLE_PERMISSIONS.IS_ALLOWED"
            + " FROM UM_ROLE_PERMISSIONS, UM_PERMISSIONS, UM_ROLES"
            + " WHERE UM_ROLE_PERMISSIONS.ROLE_ID=UM_ROLES.ID"
            + " AND UM_ROLE_PERMISSIONS.PERMISSION_ID=UM_PERMISSIONS.ID"
            + " AND UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=? AND UM_ROLES.ROLE_NAME=?";

    public static final String GET_USER_AUTHORIZED_SQL = "SELECT UM_USER_PERMISSIONS.IS_ALLOWED"
            + " FROM UM_USER_PERMISSIONS, UM_PERMISSIONS"
            + " WHERE UM_USER_PERMISSIONS.PERMISSION_ID=UM_PERMISSIONS.ID"
            + " AND UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?" + " AND UM_USER_PERMISSIONS.USER_ID=?";

    public static final String GET_ALLOWED_ROLES_FOR_RESOURCE_SQL = "SELECT UM_ROLES.ROLE_NAME"
            + " FROM UM_ROLE_PERMISSIONS, UM_PERMISSIONS, UM_ROLES"
            + " WHERE UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?"
            + " AND UM_PERMISSIONS.ID=UM_ROLE_PERMISSIONS.PERMISSION_ID"
            + " AND UM_ROLE_PERMISSIONS.ROLE_ID=UM_ROLES.ID"
            + " AND UM_ROLE_PERMISSIONS.IS_ALLOWED=1";

    public static final String GET_DENIED_ROLES_FOR_RESOURCE_SQL = "SELECT UM_ROLES.ROLE_NAME"
            + " FROM UM_ROLE_PERMISSIONS, UM_PERMISSIONS, UM_ROLES"
            + " WHERE UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?"
            + " AND UM_PERMISSIONS.ID=UM_ROLE_PERMISSIONS.PERMISSION_ID"
            + " AND UM_ROLE_PERMISSIONS.ROLE_ID=UM_ROLES.ID"
            + " AND UM_ROLE_PERMISSIONS.IS_ALLOWED=0";

    public static final String GET_ALLOWED_USERS_ON_RESOURCE_SQL = "SELECT UM_USER_PERMISSIONS.USER_ID"
            + " FROM UM_USER_PERMISSIONS, UM_PERMISSIONS"
            + " WHERE UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?"
            + " AND UM_PERMISSIONS.ID=UM_USER_PERMISSIONS.PERMISSION_ID"
            + " AND UM_USER_PERMISSIONS.IS_ALLOWED=1";

    public static final String GET_DENIED_USERS_ON_RESOURCE_SQL = "SELECT UM_USER_PERMISSIONS.USER_ID"
            + " FROM UM_USER_PERMISSIONS, UM_PERMISSIONS"
            + " WHERE UM_PERMISSIONS.RESOURCE_ID=?"
            + " AND UM_PERMISSIONS.ACTION=?"
            + " AND UM_PERMISSIONS.ID=UM_USER_PERMISSIONS.PERMISSION_ID"
            + " AND UM_USER_PERMISSIONS.IS_ALLOWED=0";

    public static final String GET_ROLE_PERMISSION_SQL = "SELECT * FROM UM_ROLE_PERMISSIONS"
            + " WHERE PERMISSION_ID=?";
    public static final String GET_USER_PERMISSION_SQL = "SELECT * FROM UM_USER_PERMISSIONS"
            + " WHERE PERMISSION_ID=?";
    public static final String GET_ROLES_ALL_SQL = "SELECT ROLE_NAME FROM UM_ROLES";
    public static final String GET_USERS_ALL_SQL = "SELECT USER_NAME FROM UM_USERS";
 //   public static final String GET_USER_ID_SQL = "SELECT ID FROM UM_USERS WHERE USER_NAME=?";
    public static final String GET_ROLE_ID_SQL = "SELECT ID FROM UM_ROLES WHERE ROLE_NAME=?";

    public static final String GET_RESOURCE_PERMISSION_SQL = "SELECT * FROM UM_PERMISSIONS"
            + " WHERE UM_PERMISSIONS.RESOURCE_ID=?";
    public static final String GET_ATTRIBUTE_NAMES_SQL = "SELECT DISTINCT ATTR_NAME"
            + " FROM  UM_USER_ATTRIBUTES";

    public static final String GET_USERS_IN_ROLE_SQL = "SELECT UM_USER_ROLES.USER_ID"
            + " FROM UM_USER_ROLES, UM_ROLES"
            + " WHERE UM_ROLES.ID=UM_USER_ROLES.ROLE_ID"
            + " AND UM_ROLES.ROLE_NAME=?";

    public static final String GET_USERS_WITH_PROPERTY_SQL = "SELECT USER_NAME"
            + " FROM UM_USERS, UM_USER_ATTRIBUTES WHERE UM_USERS.ID =UM_USER_ATTRIBUTES.USER_ID"
            + " AND UM_USER_ATTRIBUTES.ATTR_NAME=? AND UM_USER_ATTRIBUTES.ATTR_VALUE=?";

    public static final String GET_USERS_WITH_PROPERTY_VALUE_SQL = "SELECT USER_NAME"
            + " FROM UM_USERS, UM_USER_ATTRIBUTES WHERE UM_USERS.ID =UM_USER_ATTRIBUTES.USER_ID"
            + " AND UM_USER_ATTRIBUTES.ATTR_VALUE=?";
    
    public static final String GET_USERS_WITH_PROPERTY_LIKE_SQL = "SELECT USER_NAME"
        + " FROM UM_USERS, UM_USER_ATTRIBUTES WHERE UM_USERS.ID =UM_USER_ATTRIBUTES.USER_ID"
        + " AND UM_USER_ATTRIBUTES.ATTR_NAME LIKE ? AND UM_USER_ATTRIBUTES.ATTR_VALUE=?";

    public static final String COLUMN_NAME_ATTR_NAME = "attr_name";
    public static final String COLUMN_NAME_ATTR_VALUE = "attr_value";
    public static final String COLUMN_NAME_ID = "id";
    public static final String COLUMN_NAME_USER_NAME = "user_id";
    public static final String COLUMN_NAME_ROLE_NAME = "role_name";
    public static final String COLUMN_NAME_ROLE_ID = "role_id";
    public static final String COLUMN_NAME_USER_ID = "user_id";
    public static final String COLUMN_NAME_IS_ALLOWED = "is_allowed";
    public static final String COLUMN_NAME_CREDENTIAL = "USER_PASSWORD";
    public static final String COLUMN_NAME_ACTION = "action";

}
