package org.wso2.registry.users.hybrid.ldap;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.naming.directory.DirContext;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.users.UserStoreAdmin;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.def.DefaultAuthorizer;
import org.wso2.registry.users.def.util.DefaultDatabaseUtil;
import org.wso2.registry.users.hybrid.HybridRealmConstants;

public class LDAPUserStoreAdmin extends LDAPUserStoreReader implements UserStoreAdmin{

    
    private static Log log = LogFactory.getLog(DefaultAuthorizer.class);
    
    public LDAPUserStoreAdmin(LDAPRealmConfig config, DataSource dataSource, DirContext context) {
        super(config, dataSource, context);
    }
    
    public void setUserProperties(String userName, Map properties) throws UserStoreException {
        // Can do need time
    }

    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }

    public void updateUser(String userName, Object newCredential) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
        
    }

    public void addUser(String userName, Object credential) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }
    
    public void deleteUser(String userName) throws UserStoreException {
        throw new UserStoreException("Unsupported operation for this user store type");
    }

//default style
    
    public void deleteRole(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement deleteRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_ROLE_SQL);
            deleteRoleStmt.setString(1, roleName);
            deleteRoleStmt.execute();
            dbConnection.commit();
            deleteRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void addUserToRole(String userName, String roleName) throws UserStoreException {

        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

          
            int roleid = DefaultDatabaseUtil.getRoleId(dbConnection, roleName);

            if (roleid == -1) {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }

            PreparedStatement addUserToRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_USER_ROLE_SQL);
            addUserToRoleStmt.setString(1, userName);
            addUserToRoleStmt.setInt(2, roleid);
            addUserToRoleStmt.executeUpdate();
            dbConnection.commit();
            addUserToRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorAddingUserToRole"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void removeUserFromRole(String userName, String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement removeUserFromRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_USER_ROLE_SQL);
            removeUserFromRoleStmt.setString(1, userName);
            removeUserFromRoleStmt.setString(2, roleName);
            removeUserFromRoleStmt.executeUpdate();
            dbConnection.commit();
            removeUserFromRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void addRole(String roleName) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement addRoleStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_ROLE_SQL);
            addRoleStmt.setString(1, roleName);
            addRoleStmt.execute();
            dbConnection.commit();
            addRoleStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

  
    public void setRoleProperties(String roleName, Map properties) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            int roleid = DefaultDatabaseUtil.getRoleId(dbConnection, roleName);
            if (roleid == -1) {
                throw new UserStoreException(Messages.getMessage("nullRole"));
            }

            PreparedStatement deleteRolePropsStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.DELETE_ROLE_ATTRIBUTE_SQL);
            deleteRolePropsStmt.setInt(1, roleid);
            deleteRolePropsStmt.executeUpdate();

            PreparedStatement setRolePropertiesStmt = dbConnection
                    .prepareStatement(HybridRealmConstants.ADD_ROLE_ATTRIBUTE_SQL);

            Iterator ite = properties.entrySet().iterator();
            while (ite.hasNext()) {
                Entry entry = (Entry) ite.next();
                String key = (String) entry.getKey();
                String value = (String) entry.getValue();
                if (value != null) {
                    setRolePropertiesStmt.setString(1, key);
                    setRolePropertiesStmt.setString(2, value);
                    setRolePropertiesStmt.setInt(3, roleid);
                    setRolePropertiesStmt.executeUpdate();
                }
            }
            dbConnection.commit();

            deleteRolePropsStmt.close();
            setRolePropertiesStmt.close();

        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }


}
