/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry;

import junit.framework.TestCase;

import java.util.Properties;

import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.registry.app.RemoteRegistryFactory;
import org.wso2.registry.app.RemoteRegistry;
import org.wso2.registry.exceptions.RegistryException;

public class FactoryTest extends TestCase {
    public void testFactory() throws Exception {
        // First confirm that a totally default config works.
        Registry reg = RegistryFactory.newInstance().getRegistry();
        assertNotNull(reg);
        assertTrue(reg instanceof InMemoryJDBCRegistry);

        // Now try one with properties
        Properties props = new Properties();
        props.setProperty(RegistryFactory.FACTORY_CLASS_PROPERTY,
                          "org.wso2.registry.app.RemoteRegistryFactory");
        boolean caughtException = false;
        try {
            RegistryFactory.newInstance(props).getRegistry();
        } catch (RegistryException e) {
            // Awesome, this was supposed to fail (no URL)
            caughtException = true;
        }
        assertTrue(caughtException);

        props.setProperty(RemoteRegistryFactory.REMOTE_URL,
                          "http://localhost:8080/wso2registry");
        reg = RegistryFactory.newInstance(props).getRegistry();
        assertNotNull(reg);
        assertTrue(reg instanceof RemoteRegistry);
    }

    public void testPropertyFile() throws Exception {
        Registry reg = RegistryFactory.newInstance("src/test/resources/regtest.properties").getRegistry();
        assertNotNull(reg);
        assertTrue(reg instanceof RemoteRegistry);
    }
}
