/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.realm.RegistryRealm;

import java.util.Map;
import java.util.HashMap;

public class CommentsTest extends TestCase {

    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() throws RegistryException {
        if (registry == null) {
            InMemoryEmbeddedRegistry embeddedRegistry = new InMemoryEmbeddedRegistry();
            registry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
        }
    }

    public void testCommentQuery() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setContent("r1 content");
        registry.put("/test/comments/r1", r1);

        registry.addComment("/test/comments/r1", new Comment("comment1 on this resource :)"));
        registry.addComment("/test/comments/r1", new Comment("comment2 on this resource :)"));

        Resource comQuery = registry.newResource();
        String sql = "SELECT COMMENT_ID FROM REG_COMMENT WHERE COMMENT_TEXT LIKE ?";
        comQuery.setContent(sql);

        comQuery.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
        comQuery.addProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                RegistryConstants.COMMENTS_RESULT_TYPE);
        
        registry.put("/test/comments/q1", comQuery);

        Map<String, String> params = new HashMap <String, String> ();
        params.put("1", "comment1%");
        Collection qResults = registry.executeQuery("/test/comments/q1", params);

        String[] qPaths = (String[]) qResults.getContent();

        assertEquals("Query result count should be 1", qPaths.length, 1);

        Resource qResult = registry.get(qPaths[0]);

        assertTrue("Comment query should return Comment objects as results",
                qResult instanceof Comment);

        Comment c1 = (Comment) qResult;
        
        assertNotNull("Comment query result is invalid", c1.getText());
        assertNotNull("Comment query result is invalid", c1.getUser());
        assertNotNull("Comment query result is invalid", c1.getCreatedTime());
    }

    public void testCommentDelete() throws RegistryException {

        String r1Path = "/c1d1/c1";
        Collection r1 = registry.newCollection();
        registry.put(r1Path, r1);

        String c1Path = registry.addComment(r1Path, new Comment("test comment1"));
        String c2Path = registry.addComment(r1Path, new Comment("test comment2"));

        Comment[] comments1 = registry.getComments(r1Path);

        assertEquals("There should be two comments.", comments1.length, 2);

        String[] cTexts1 = {comments1[0].getText(), comments1[1].getText()};

        assertTrue("comment is missing", containsString(cTexts1, "test comment1"));
        assertTrue("comment is missing", containsString(cTexts1, "test comment2"));

        registry.delete(c1Path);

        Comment[] comments2 = registry.getComments(r1Path);

        assertEquals("There should be one comment.", 1, comments2.length);

        String[] cTexts2 = {comments2[0].getText()};

        assertTrue("comment is missing", containsString(cTexts2, "test comment2"));
        assertTrue("deleted comment still exists", !containsString(cTexts2, "test comment1"));
    }


    private boolean containsString(String[] array, String value) {

        boolean found = false;
        for (String anArray : array) {
            if (anArray.startsWith(value)) {
                found = true;
                break;
            }
        }

        return found;
    }
}
