/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.Collection;
import org.wso2.registry.users.UserStoreAdmin;
import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.utils.RegistryUtils;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.realm.RegistryRealm;

import java.io.InputStream;
import java.io.FileInputStream;
import java.util.*;

public class MoveTest extends TestCase {

    protected static EmbeddedRegistry embeddedRegistry = null;
    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() throws Exception {
        if (registry == null) {
            embeddedRegistry = new InMemoryEmbeddedRegistry();
            registry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
        }
    }

    public void testResourceMoveFromRoot() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setProperty("test", "move");
        r1.setContent("c");
        registry.put("/move1", r1);

        Collection c1 = registry.newCollection();
        registry.put("/test/move", c1);

        registry.move("/move1", "/test/move/move1");

        Resource newR1 = registry.get("/test/move/move1");
        assertEquals("Moved resource should have a property named 'test' with value 'move'.",
                newR1.getProperty("test"), "move");

        boolean failed = false;
        try {
            Resource oldR1 = registry.get("/move1");
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("Moved resource should not be accessible from the old path.", failed);
    }

    public void testResourceMoveToRoot() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setProperty("test", "move");
        r1.setContent("c");
        registry.put("/test/move/move2", r1);

        registry.move("/test/move/move2", "/move2");

        Resource newR1 = registry.get("/move2");
        assertEquals("Moved resource should have a property named 'test' with value 'move'.",
                newR1.getProperty("test"), "move");

        boolean failed = false;
        try {
            Resource oldR1 = registry.get("/test/move/move2");
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("Moved resource should not be accessible from the old path.", failed);
    }

    public void testGeneralResourceMove() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setProperty("test", "move");
        r1.setContent("c");
        registry.put("/test/c1/move/move3", r1);

        Collection c2 = registry.newCollection();
        registry.put("/test/c2/move", c2);

        registry.move("/test/c1/move/move3", "/test/c2/move/move3");

        Resource newR1 = registry.get("/test/c2/move/move3");
        assertEquals("Moved resource should have a property named 'test' with value 'move'.",
                newR1.getProperty("test"), "move");

        boolean failed = false;
        try {
            Resource oldR1 = registry.get("/test/c1/move/move3");
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("Moved resource should not be accessible from the old path.", failed);
    }

    public void testGeneralCollectionMove() throws RegistryException {

        Resource r1 = registry.newResource();
        r1.setProperty("test", "move");
        r1.setContent("c");
        registry.put("/test/c1/move5/move/dummy", r1);

        Collection c2 = registry.newCollection();
        registry.put("/test/c3", c2);

        registry.move("/test/c1/move5", "/test/c3/move5");

        Resource newR1 = registry.get("/test/c3/move5/move/dummy");
        assertEquals("Moved resource should have a property named 'test' with value 'move'.",
                newR1.getProperty("test"), "move");

        boolean failed = false;
        try {
            Resource oldR1 = registry.get("/test/c1/move5/move/dummy");
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("Moved resource should not be accessible from the old path.", failed);
    }
}
