/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.realm.RegistryRealm;

public class PaginationTest extends TestCase {

    protected static EmbeddedRegistry embeddedRegistry = null;
    protected static Registry registry = null;
    protected static RegistryRealm realm = null;

    public void setUp() throws RegistryException {
        if (registry == null) {
            embeddedRegistry = new InMemoryEmbeddedRegistry();
            registry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);
        }
    }

    public void testCollectionPagination() throws RegistryException {
        Collection c1 = registry.newCollection();
        registry.put("/test/c1", c1);
        for (int j = 0; j < 50; j++) {
            Collection ci = registry.newCollection();
            registry.put(String.format("/test/c1/c_%02d", j), ci);
        }
        Resource collection = registry.get("/test/c1");
        String childNodes[] = (String[])collection.getContent();
        assertEquals(50, childNodes.length);
        Collection coll = registry.get("/test/c1", 0, 20);
        assertEquals(20, coll.getChildCount());
        childNodes = coll.getChildren();
        assertEquals(20, childNodes.length);
        coll = (Collection)registry.get("/test/c1");
        assertEquals(50, coll.getChildCount());

        coll = registry.get("/test/c1", 20, 5);
        childNodes = coll.getChildren();
        assertEquals(5, childNodes.length);
        assertTrue(childNodes[0].indexOf("20") > -1);
    }
}
