/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.secure;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.InMemoryEmbeddedRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.users.UserRealm;

public class CommentsTest extends TestCase {

    private static EmbeddedRegistry registry = null;

    public void setUp() throws RegistryException {
        if (registry == null) {
            registry = new InMemoryEmbeddedRegistry();
        }
    }

    public void testCommentDelete() throws Exception {

        UserRegistry adminReg = registry.
                getUserRegistry(RegistryConstants.ADMIN_USER, RegistryConstants.ADMIN_PASSWORD);

        UserRealm adminRealm = adminReg.getUserRealm();
        adminRealm.getUserStoreAdmin().addUser("cu1", "cu1");

        UserRegistry cu1Reg = registry.getUserRegistry("cu1", "cu1");

        ResourceImpl r1 = new ResourceImpl();
        r1.setContent("r1 content");
        adminReg.put("/crtest/r1", r1);

        adminRealm.getAccessControlAdmin().
                authorizeUser("cu1", "/crtest/r1", ActionConstants.PUT);

        ResourceImpl r2 = new ResourceImpl();
        r2.setContent("r2 content");
        adminReg.put("/crtest/r2", r2);

        String c1 =
                adminReg.addComment("/crtest/r1", new Comment("admin's comment on /crtest/r1"));
        System.out.println(adminRealm.getAccessControlAdmin().isUserAuthorized("cu1", "/crtest/r1", ActionConstants.PUT));

        String c2 =
                adminReg.addComment("/crtest/r2", new Comment("admin's comment on /crtest/r2"));

        String c3 =
                cu1Reg.addComment("/crtest/r1", new Comment("cu1's comment on /crtest/r1"));

        String c4 =
                cu1Reg.addComment("/crtest/r2", new Comment("cu1's comment on /crtest/r2"));

        cu1Reg.delete(c1);

        boolean failed = false;
        try {
            cu1Reg.delete(c2);
        } catch (RegistryException e) {
            failed = true;
        }
        assertTrue("cu1 should not have " +
                "permission to delete admin's comment on admin's resource.", failed);

        try {
            cu1Reg.delete(c3);
        } catch (RegistryException e) {
            fail("cu1 should be able to delete cu1's comment on admin's resource even he does" +
                    " not have write permissions for it.");
        }

        try {
            cu1Reg.delete(c4);
        } catch (RegistryException e) {
            fail("admin should be able to delete cu1's comment on admin's resource. In this case" +
                    "cu1 also has write permissions on admin's resource.");
        }

    }
}
