/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.actions.AddCollectionAction;

import javax.servlet.http.HttpServletRequest;

public class ActionInvoker {

    public static final String SUCCESS = "SUCCESS";
    public static final String ERROR = "ERROR";

    public String invokeActions(HttpServletRequest request) {

        String contextRoot = getContextRoot(request);

        String regAction = request.getParameter("regAction");
        String path = (String) request.getSession().getAttribute(RegistryConstants.PATH_ATTR);

        if (regAction != null) {
            if (regAction.equals("addCollection")) {

                String collectionName = request.getParameter("collectionName");
                String mediaType = request.getParameter("mediaType");
                String description = request.getParameter("description");
                AddCollectionAction addCollectionAction = new AddCollectionAction();
                addCollectionAction.setParentPath(path);
                addCollectionAction.setCollectionName(collectionName);
                addCollectionAction.setMediaType(mediaType);
                addCollectionAction.setDescription(description);

                try {
                    addCollectionAction.execute(request);
                } catch (RegistryException e) {
                    e.printStackTrace();
                    return ERROR;
                }
            }
        }

        return SUCCESS;
    }

    public String getContextRoot(HttpServletRequest req) {

        String contextPath = req.getContextPath();
        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        
        return contextPath;
    }
}
