/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.config.RegistryContext;
import org.wso2.registry.jdbc.handlers.CustomEditManager;
import org.wso2.registry.utils.RegistryUtils;
import org.wso2.registry.utils.VersionedPath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.builders.ResourcePageBuilder;
import org.wso2.registry.web.utils.CommonUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;
import java.net.URLDecoder;

public class CustomUIServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(CustomUIServlet.class);

    private String contextRoot;

    private int serverPrefixLength;

    private CustomEditManager customEditManager;

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);

        customEditManager = RegistryContext.getSingleton().getCustomEditManager();
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();

        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;

        serverPrefixLength = contextRoot.length() +
                RegistryConstants.PATH_SEPARATOR.length() + UIConstants.CUSTOM_PATH.length();
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        try {
            customEditManager.process(request, response);
        } catch (RegistryException e) {
            String msg = "Failed to execute custom UI processing. " + e.getMessage();
            log.error(msg, e);
            CommonUtil.addErrorMessage(request, msg);
        }
    }
}
