/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.builders.PeoplePageBuilder;
import org.wso2.registry.web.builders.UserPageBuilder;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;
import java.net.URLDecoder;

/**
 * This servlet is responsible for generating the people page (/wso2registry/people) and user
 * pages (/wso2registry/people/user1).
 */
public class PeopleServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(PeopleServlet.class);

    private String contextRoot;

    private int serverPrefixLength;

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();

        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;

        serverPrefixLength = contextRoot.length() + RegistryConstants.PATH_SEPARATOR.length() +
                UIConstants.PEOPLE_PATH.length() + RegistryConstants.PATH_SEPARATOR.length();
    }

    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        String url = URLDecoder.decode(request.getRequestURI(), "UTF-8");

        // if URL is /wso2registry/people or /wso2registry/people/ go to people page
        if (url.length() <= serverPrefixLength) {
            PeoplePageBuilder.build(request, response);
        } else {
            String user = url.substring(serverPrefixLength, url.length());
            UserPageBuilder.build(request, response, user);
        }
    }

    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        super.doPost(request, response);
    }
}
