/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.web.ConsoleConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

public abstract class AbstractRegistryAction {

    public static final String SUCCESS = "SUCCESS";
    public static final String ERROR = "ERROR";

    protected HttpServletRequest request = null;

    public void setRequest(HttpServletRequest request) {
        this.request = request;
    }

    protected Registry getRegistry() throws RegistryException {

        Registry registry =
                (Registry) request.getSession().getAttribute(ConsoleConstants.USER_REGISTRY);

        if (registry == null) {

            // user is not logged in. create a annoymous registry for the user.

            ServletContext context =
                    request.getSession().getServletContext();
            EmbeddedRegistry embeddedRegistry =
                    (EmbeddedRegistry)context.getAttribute(RegistryConstants.REGISTRY);

            UserRegistry userRegistry = embeddedRegistry.getUserRegistry();

            request.getSession().setAttribute(ConsoleConstants.USER_REGISTRY, userRegistry);

            registry = userRegistry;
        }

        return registry;
    }

    public String getServerBaseURL() {
        String reqURL = request.getRequestURL().toString();
        return reqURL.substring(0, reqURL.indexOf("/wso2registry"));
    }

    public boolean isLoggedIn() throws RegistryException {

        Registry registry = getRegistry();
        if (registry instanceof UserRegistry) {
            UserRegistry userRegistry = (UserRegistry)registry;
            String userID = userRegistry.getUserName();

            if (!userID.equals(RegistryConstants.ANONYMOUS_USER)) {
                return true;
            }
        }

        return false;
    }

    public String getUserName() throws RegistryException {

        Registry registry = getRegistry();
        if (registry instanceof UserRegistry) {
            UserRegistry userRegistry = (UserRegistry)registry;
            return userRegistry.getUserName();
        }

        return null;
    }
}
