/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;

import javax.servlet.http.HttpServletRequest;

public class AjaxRatingAction extends AbstractRegistryAction {

    private String resourcePath;
    private int userRating = -1;
    private float averageRating;

    private String[] userStars = new String[5];
    private String[] averageStars = new String[5];

    private boolean versionView = false;

    public void execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        UserRegistry userRegistry = (UserRegistry) getRegistry();

        if (userRating != -1) {
            userRegistry.rateResource(resourcePath, userRating);
        }

        averageRating = userRegistry.getAverageRating(resourcePath);

        // round the average rating to closest 3 decimal points
        float tempRating = averageRating * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        averageRating = tempRating;

        userRating = userRegistry.getRating(resourcePath, userRegistry.getUserName());

        for (int i = 0; i < 5; i++) {

            if (userRating >= i + 1) {
                userStars[i] = "04";

            } else if (userRating <= i) {
                userStars[i] = "00";

            }
        }

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        String[] q = resourcePath.split("\\;");
        if (q.length == 2) {
            if (q[1].startsWith("version=")) {
                versionView = true;
            }
        }
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public int getUserRating() {
        return userRating;
    }

    public void setUserRating(int userRating) {
        this.userRating = userRating;
    }

    public float getAverageRating() {
        return averageRating;
    }

    public void setAverageRating(float averageRating) {
        this.averageRating = averageRating;
    }

    public String[] getUserStars() {
        return userStars;
    }

    public void setUserStars(String[] userStars) {
        this.userStars = userStars;
    }

    public String[] getAverageStars() {
        return averageStars;
    }

    public void setAverageStars(String[] averageStars) {
        this.averageStars = averageStars;
    }

    public boolean isVersionView() {
        return versionView;
    }
}
