package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.utils.ResourceData;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class AssociationTreeViewAction extends AbstractRegistryAction {
    private String resourcePath;
    private String assoType;
    private String associationTree = "";
    private int assoIndex = 0;          // Use to give an unique id for the generated html elements
    private List<String> treeCache = new ArrayList<String>();

    public void execute(HttpServletRequest request) throws RegistryException {
        setRequest(request);

        List fatherAsso = provideChildAssociations(resourcePath);
        Iterator ifatherAssofaterAsso = fatherAsso.iterator();
        while (ifatherAssofaterAsso.hasNext() && fatherAsso != null) {
            Association asso = (Association) ifatherAssofaterAsso.next();
            createAssociationTree(asso.getDestinationPath(), asso.getAssociationType());
        }
    }


    public void createAssociationTree(String path, String type) {
        assoIndex++;
        Registry registry;
        try {
            registry = getRegistry();

            Resource father = registry.get(path);

            String fatherResourceType = father instanceof Collection ? ResourceData.COLLECTION : ResourceData.RESOURCE;
            List fatherAsso = provideChildAssociations(path);
            treeCache.add(path);

            boolean foundInfinite = false;

            if (fatherAsso == null) foundInfinite = true;

            associationTree += "<div class=\"father-object\" >";
            associationTree += "<ul class=\"tree-row-object\"><li class=\"first\">";
            if (!foundInfinite) {
                if (!fatherAsso.isEmpty()) {
                    associationTree += "<a onclick=\"showHideCommon('y_" + assoIndex + "');showHideCommon('xminus_" + assoIndex + "');showHideCommon('xplus_" + assoIndex + "');\">" +
                            "<img src=\"/wso2registry/admin/images/icon-tree-minus.jpg\" id=\"xminus_" + assoIndex + "\" style=\"margin-right:2px;\" />" +
                            "<img src=\"/wso2registry/admin/images/icon-tree-plus.jpg\" id=\"xplus_" + assoIndex + "\" style=\"margin-right:2px;display:none;\" />" +
                            "</a>";
                } else {
                    associationTree += "<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:15px;\" />";
                }
            } else {
                associationTree += "<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:15px;\" />";
            }
            if (fatherResourceType.equals(ResourceData.COLLECTION)) {
                associationTree += "<img src=\"/wso2registry/admin/images/icon-folder-small.gif\" style=\"margin-right:2px;\" />";
            } else {
                associationTree += "<img src=\"/wso2registry/admin/images/editshred.png\" style=\"margin-right:2px;\" />";
            }

            associationTree += "<a href=\"" + calculateRelativePath(path) + "\">";
            if (foundInfinite) {
                associationTree += path + "(infinite loop..)";
            } else {
                associationTree += path;
            }
            associationTree += "</a></li>";
            if (!assoType.equals(UIConstants.ASSOCIATION_TYPE01))
                associationTree += "<li class=\"second\">" + type + "</li>";
            associationTree += "</ul></div>";
            if (!foundInfinite && !fatherAsso.isEmpty()) {
                associationTree += "<div class=\"child-objects\" id=\"y_" + assoIndex + "\">";
                Iterator iFaterAsso = fatherAsso.iterator();
                while (iFaterAsso.hasNext()) {
                    Association asso = (Association) iFaterAsso.next();

                    // if path is equal to the destination path, it is a backward association. we are only displaying
                    // the forward associations here
                    if (!path.equals(asso.getDestinationPath())) {
                        if(!treeCache.contains(asso.getDestinationPath())){
                            createAssociationTree(asso.getDestinationPath(), asso.getAssociationType());
                        }
                    }
                }
                associationTree += "</div>";

            }
        } catch (RegistryException e) {
            e.printStackTrace();
        }


    }

    public String calculateRelativePath(String resourcePath) {
        if (resourcePath.startsWith("http://")) {
            return resourcePath;
        }

        String relativePath = resourcePath;

        if (RegistryConstants.ROOT_PATH.equals(resourcePath)) {
            relativePath = "";
        } else {
            if (resourcePath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                relativePath = resourcePath.substring(1, resourcePath.length());
            }
        }
        return relativePath;
    }

    public List provideChildAssociations(String resourcePath) throws RegistryException {
        List tmpAssociations = new ArrayList();
        List associations = new ArrayList();
        Registry registry = getRegistry();
        Resource resource = registry.get(resourcePath);
        Association[] asso = registry.getAllAssociations(resource.getPath());

        tmpAssociations.addAll(Arrays.asList(asso));

        Iterator iAssociations = tmpAssociations.iterator();
        while (iAssociations.hasNext()) {
            Association tmpAsso = (Association) iAssociations.next();
            //Get all associations filtered by it's association type
            if (!resourcePath.equals(tmpAsso.getDestinationPath())) {
                if (tmpAsso.getAssociationType().equals(UIConstants.ASSOCIATION_TYPE01) && assoType.equals(UIConstants.ASSOCIATION_TYPE01))
                    associations.add(tmpAsso);
                if (!tmpAsso.getAssociationType().equals(UIConstants.ASSOCIATION_TYPE01) && !assoType.equals(UIConstants.ASSOCIATION_TYPE01))
                    associations.add(tmpAsso);
            }
        }
        // if path is equal to the destination path, it is a backward association. we are only displaying
        // the forward associations here


        return associations;

    }

    public String getAssoType() {
        if (assoType.equals(UIConstants.ASSOCIATION_TYPE01)) assoType = "Dependency";
        return assoType;
    }

    public void setAssoType(String assoType) {
        this.assoType = assoType;
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getAssociationTree() {
        return associationTree;
    }

    public void setAssociationTree(String associationTree) {
        this.associationTree = associationTree;
    }

}
