/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.servlet.utils.UserUtil;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.ResourceData;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

public class CollectionPagesViewAction extends AbstractRegistryAction {

//    private Log log = LogFactory.getLog(CollectionViewAction.class);

    private List resourceDataList = new ArrayList();
    private String[] childPaths;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        // get the resource for the path
        // get all its children and fill ResourceOverview objects
        // add ResourceOverview object to the resourceDataList list

        Registry registry = getRegistry();

        for (String childPath : childPaths) {

            try {
                Resource child = registry.get(childPath);

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(childPath); // + RegistryConstants.VIEW_ACTION);

                if (childPath != null) {
                    String[] parts = childPath.split(RegistryConstants.PATH_SEPARATOR);
                    resourceData.setName(parts[parts.length - 1]);
                }

                resourceData.setResourceType(child instanceof Collection ?
                        ResourceData.COLLECTION : ResourceData.RESOURCE);
                resourceData.setAuthorUserName(child.getAuthorUserName());
                resourceData.setDescription(child.getDescription());
                resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                resourceData.setCreatedOn(child.getCreatedTime());
                resourceData.setDeleteAllowed(
                        UserUtil.isDeleteAllowed(getUserName(), childPath, request));
                calculateAverageStars(resourceData);

                resourceDataList.add(resourceData);

            } catch (AuthorizationFailedException ignore) {
                // if we get an auth failed exception while accessing a child, we simply skip it.
                // we are not showing unauthorized resources.
            }


        }

        return SUCCESS;
    }

    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    public String[] getChildPaths() {
        return childPaths;
    }

    public void setChildPaths(String[] childPaths) {
        this.childPaths = childPaths;
    }
    private void calculateAverageStars(ResourceData resourceData) {

        float tempRating = resourceData.getAverageRating() * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        resourceData.setAverageRating(tempRating);

        float averageRating = resourceData.getAverageRating();
        String[] averageStars = new String[5];

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        resourceData.setAverageStars(averageStars);
    }
}
