/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.LogEntry;
import org.wso2.registry.LogEntryCollection;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class RecentActivityAction extends AbstractRegistryAction {

    private String resourcePath;
    private String userName;
    private String fromDate;
    private String toDate;
    private String filter;
    private int totalCount;
    private int page;
    private int pageLen;
    private int start;

    private List activity = new ArrayList();


    public String execute(HttpServletRequest request) throws Exception {

        setRequest(request);

        pageLen = RegistryConstants.ITEMPS_PER_PAGE;
        start = (pageLen) * (page - 1);

        if (resourcePath != null && resourcePath.equals("")) {
            resourcePath = null;
        }

        if (userName != null && userName.equals("")) {
            userName = null;
        }

        if (fromDate != null && fromDate.equals("")) {
            fromDate = null;
        }

        if (toDate != null && toDate.equals("")) {
            toDate = null;
        }

        if (filter == null) {
            filter = "1";
        }

        int filterValue = -1;
        if (filter.equals("resourceAdd")) {
            filterValue = LogEntry.ADD;

        } else if (filter.equals("resourceUpdate")) {
            filterValue = LogEntry.UPDATE;

        } else if (filter.equals("commentings")) {
            filterValue = LogEntry.COMMENT;

        } else if (filter.equals("taggings")) {
            filterValue = LogEntry.TAG;

        } else if (filter.equals("ratings")) {
            filterValue = LogEntry.RATING;

        } else if (filter.equals("restore")) {
            filterValue = LogEntry.RESTORE;

        } else if (filter.equals("delete")) {
            filterValue = LogEntry.DELETE_RESOURCE;
        }

        UserRegistry userRegistry = (UserRegistry) getRegistry();
        //LogEntryCollection logsCollection = userRegistry.getLogCollection(resourcePath, filterValue, userName, computeDate(fromDate), computeDate(toDate), true);

        //LogEntry[] totalLogs =userRegistry.getLogs(resourcePath, filterValue, userName, computeDate(fromDate), computeDate(toDate), true);
        //totalCount = totalLogs.length;

        //LogEntry[] logs = logsCollection.getLogEntries(start, pageLen);
        LogEntry[] logs =userRegistry.getLogs(resourcePath, filterValue, userName, computeDate(fromDate), computeDate(toDate), true);
        for (int i = 0; i < logs.length; i++) {

            LogEntry logEntry = logs[i];

            if (logEntry.getAction() == LogEntry.ADD) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has added the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.UPDATE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has updated the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.DELETE_RESOURCE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has deleted the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.COMMENT) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has commented on resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + " with the following comment.<br/>" + logEntry.getActionData();
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.TAG) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has tagged the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> with tag '" + logEntry.getActionData() + "' on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.RATING) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has rated the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> with rating " + logEntry.getActionData() + " on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.RESTORE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has restored the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> to the version " + logEntry.getActionData() + " on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);
            }
        }

        return SUCCESS;
    }
    public int getTotalCount() {
        return totalCount;
    }

    public void setTotalCount(int totalCount) {
        this.totalCount = totalCount;
    }
    public int getPageLen() {
        return pageLen;
    }

    public void setPageLen(int pageLen) {
        this.pageLen = pageLen;
    }


    public int getStart() {
        return start;
    }

    public void setStart(int start) {
        this.start = start;
    }

    public int getPage() {
        return page;
    }

    public void setPage(int page) {
        this.page = page;
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getFromDate() {
        return fromDate;
    }

    public void setFromDate(String fromDate) {
        this.fromDate = fromDate;
    }

    public String getToDate() {
        return toDate;
    }

    public void setToDate(String toDate) {
        this.toDate = toDate;
    }

    public String getFilter() {
        return filter;
    }

    public void setFilter(String filter) {
        this.filter = filter;
    }

    public List getActivity() {
        return activity;
    }

    public void setActivity(List activity) {
        this.activity = activity;
    }

    /**
     * Converts given strings to Dates
     *
     * @param dateString Allowed format mm/dd/yyyy
     * @return Date corresponding to the given string date
     */
    private Date computeDate(String dateString) throws RegistryException {

        if (dateString == null || dateString.length() == 0) {
            return null;
        }

        DateFormat formatter = DateFormat.getDateInstance(DateFormat.SHORT);
        try {
            return formatter.parse(dateString);
        } catch (ParseException e) {
            String msg = "Date format is invalid: " + dateString;
            throw new RegistryException(msg, e);
        }
    }
}
