/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

public class ResourceHandler {

    public static void fetchURLResource(HttpServletRequest request) throws RegistryException {

        String parentPath = request.getParameter("path");
        String resourceName = request.getParameter("filename");

        String resourcePath;
        if (RegistryConstants.ROOT_PATH.equals(parentPath)) {
            resourcePath = RegistryConstants.ROOT_PATH + resourceName;
        } else {
            resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
        }

        String fetchURL = request.getParameter("fetchURL");
        String mediaType = request.getParameter("mediaType");
        String description = request.getParameter("description");

        UserRegistry userRegistry = getUserRegistry(request);
        Resource metadata = userRegistry.newResource();

        metadata.setMediaType(mediaType);
        metadata.setDescription(description);

        userRegistry.importResource(resourcePath, fetchURL, metadata);
    }

    private static UserRegistry getUserRegistry(HttpServletRequest request)
            throws RegistryException {

        UserRegistry userRegistry =
                (UserRegistry)request.getSession().getAttribute(RegistryConstants.USER_REGISTRY);

        if (userRegistry == null) {

            // user is not logged in. create a annoymous userRegistry for the user.

            ServletContext context = request.getSession().getServletContext();

            EmbeddedRegistry embeddedRegistry =
                    (EmbeddedRegistry)context.getAttribute(RegistryConstants.REGISTRY);

            userRegistry = embeddedRegistry.getUserRegistry();
            request.getSession().setAttribute(RegistryConstants.USER_REGISTRY, userRegistry);
        }

        return userRegistry;
    }
}
