/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.web.ConsoleConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

public class SignInAction extends AbstractRegistryAction {

    /**
     * Home path of the user. User will be directed to this path on sign in.
     */
    private String userPath;

    private String userName;
    private String password;
    private String path = RegistryConstants.ROOT_PATH;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        if (userName.equals(RegistryConstants.SYSTEM_USER)) {
            String msg = "Users are not allowed to log in as the system user.";
            throw new RegistryException(msg);
        }

        ServletContext context = request.getSession().getServletContext();
        EmbeddedRegistry embeddedRegistry =
                (EmbeddedRegistry) context.getAttribute(RegistryConstants.REGISTRY);

        UserRegistry userRegistry = embeddedRegistry.getUserRegistry(userName, password);
        request.getSession().setAttribute(ConsoleConstants.USER_REGISTRY, userRegistry);

        this.setUserPath(RegistryConstants.ROOT_PATH);

        return SUCCESS;
    }

    public String getNewUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getUserPath() {
        return userPath;
    }

    public void setUserPath(String userPath) {
        this.userPath = userPath;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }
}
