/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.LogEntry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.Arrays;

public class UserDetailsAction extends AbstractRegistryAction {

    private String displayUserName;
    private String friendlyName = "";
    private List userActivity = new ArrayList();
    private List userRoles = new ArrayList();
    private List allRoles = new ArrayList();
    private boolean userEditable;
    private boolean userAdmin;

    public String execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        UserRegistry userRegistry = (UserRegistry) getRegistry();
        UserRealm realm = userRegistry.getUserRealm();

        if (getUserName().equals(displayUserName) &&
                !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {
            userEditable = true;
        }

        try {

            Authorizer auth = realm.getAuthorizer();
            if (auth.isUserAuthorized(
                    getUserName(), AccessControlConstants.USER_RESOURCE, AccessControlConstants.EDIT) &&
                    !RegistryConstants.SYSTEM_USER.equals(displayUserName) &&
                    !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {

                userEditable = true;
            }

            if (auth.isUserAuthorized(getUserName(),
                            AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.ADD) &&
                    auth.isUserAuthorized(getUserName(),
                            AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.DELETE) &&
                    !RegistryConstants.SYSTEM_USER.equals(displayUserName) &&
                    !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {

                userAdmin = true;
            }

            friendlyName = (String) realm.
                    getUserStoreAdmin().getUserProperties(displayUserName).get("friendlyName");
            if (friendlyName == null) {
                friendlyName = "";
            }

            String[] rolesArray = realm.getUserStoreReader().getUserRoles(displayUserName);
            userRoles.addAll(Arrays.asList(rolesArray));
        } catch (UserStoreException e) {
            String msg = "Could not get the user information. Caused by: " + e.getMessage();
            throw new RegistryException(msg, e);
        }

        try {
            String[] roleNamesArray = realm.getUserStoreReader().getAllRoleNames();
            allRoles.addAll(Arrays.asList(roleNamesArray));
        } catch (UserStoreException e) {
            String msg = "Could not get all roles. Caused by: " + e.getMessage();
            throw new RegistryException(msg);
        }

        LogEntry[] userLogs = userRegistry.getLogs(null, -1, displayUserName, null, null, true);
        for (int i = 0; i < userLogs.length && i < 10; i++) {
            LogEntry logEntry = userLogs[i];
            userActivity.add(logEntry.getText());
        }

        return SUCCESS;
    }

    public String getDisplayUserName() {
        return displayUserName;
    }

    public void setDisplayUserName(String displayUserName) {
        this.displayUserName = displayUserName;
    }

    public String getFriendlyName() {
        return friendlyName;
    }

    public List getUserActivity() {
        return userActivity;
    }

    public void setUserActivity(List userActivity) {
        this.userActivity = userActivity;
    }

    public List getUserRoles() {
        return userRoles;
    }

    public void setUserRoles(List userRoles) {
        this.userRoles = userRoles;
    }

    public List getAllRoles() {
        return allRoles;
    }

    public void setAllRoles(List allRoles) {
        this.allRoles = allRoles;
    }

    public boolean isUserEditable() {
        return userEditable;
    }

    public boolean isUserAdmin() {
        return userAdmin;
    }
}
