/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions.utils;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.web.utils.CommonUtil;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public class ResourceData {

    public static final String RESOURCE = "resource";
    public static final String COLLECTION = "collection";
    public static final String UNKNOWN = "unknown";

    private String name;
    private String resourcePath;
    private String relativePath;

    /**
     * Resource, Collection or Unknown
     * Unknown type is set for entities, to which the current user doesn't have get permissions
     */
    private String resourceType;

    private String authorUserName;
    private String description;
    private float averageRating;
    private String[] averageStars = new String[5];
    private Date createdOn;
    private boolean deleteAllowed;
    private Map tagCounts = new HashMap();

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
        String[] q = name.split("\\?");
        if (q.length == 2) {
            if (q[1].startsWith("v=")) {
                //String versionNumber = q[1].substring("v=".length());
                //name = q[0] + " (version " + versionNumber + ")";
                name = q[0];
            }
        }

        this.name = name;
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;

        if (RegistryConstants.ROOT_PATH.equals(resourcePath)) {
            relativePath = "";
        } else {
            if (resourcePath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                relativePath = resourcePath.substring(1, resourcePath.length());
            }
        }        
    }

    public String getRelativePath() {
        return relativePath;
    }

    public void setRelativePath(String relativePath) {
        this.relativePath = relativePath;
    }

    public String getResourceType() {
        return resourceType;
    }

    public void setResourceType(String resourceType) {
        this.resourceType = resourceType;
    }

    public String getAuthorUserName() {
        return authorUserName;
    }

    public void setAuthorUserName(String authorUserName) {
        this.authorUserName = authorUserName;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public float getAverageRating() {
        return averageRating;
    }

    public void setAverageRating(float averageRating) {
        this.averageRating = averageRating;
    }

    public String[] getAverageStars() {
        return averageStars;
    }

    public void setAverageStars(String[] averageStars) {
        this.averageStars = averageStars;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public String getFormattedCreatedOn() {
        return CommonUtil.formatDate(createdOn);
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public boolean isDeleteAllowed() {
        return deleteAllowed;
    }

    public void setDeleteAllowed(boolean deleteAllowed) {
        this.deleteAllowed = deleteAllowed;
    }

    public Map getTagCounts() {
        return tagCounts;
    }

    public void setTagCounts(Map tagCounts) {
        this.tagCounts = tagCounts;
    }
}
