/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.builders;

import org.wso2.registry.web.populators.*;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class ResourcePageBuilder {

    private static final Log log = LogFactory.getLog(ResourcePageBuilder.class);

    public static void build(HttpServletRequest request, HttpServletResponse response) {

        // populate all necessary beans
        // forward to the resource.jsp

        ResourcePath path = (ResourcePath) request.getSession().
                getAttribute(UIConstants.PROCESSED_PATH);

        Resource resource = null;
        UserRegistry userRegistry;
        try {
            userRegistry = CommonUtil.getUserRegistry(request);
            resource = userRegistry.get(path.getCompletePath());

        } catch (RegistryException e) {

            String msg = "Failed to get resource details of resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            CommonUtil.addErrorMessage(request, msg);

            CommonUtil.forwardToPage(request, response, UIConstants.RESOURCES_JSP);
            return;
        }

        CommonBeanPopulator.populate(request, path);
        PropertiesBeanPopulator.populate(request, resource);
        ResourceMetadataPopulator.populate(request, resource);
        ContentBeanPopulator.populate(request, resource, path);
        PermissionsBeanPopulator.populate(request, userRegistry, path);
        TagBeanPopulator.populate(request, userRegistry, resource);
        CommentBeanPopulator.populate(request, userRegistry, resource);
        LifeCycleBeanPopulator.populate(request, userRegistry, resource, path);
        DependenciesBeanPopulator.populate(request, userRegistry, path);

        resource.discard();

        CommonUtil.forwardToPage(request, response, UIConstants.RESOURCES_JSP);
    }
}
