/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.web.beans.ActivityBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.LogEntry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import java.text.DateFormat;
import java.text.ParseException;

public class ActivityBeanPopulator {

    private static final Log log = LogFactory.getLog(ActivityBeanPopulator.class);

    public static void populate(HttpServletRequest request) {

        ActivityBean activityBean = new ActivityBean();

        String userName = request.getParameter("userName");
        String resourcePath = request.getParameter("path");
        String fromDate = request.getParameter("fromDate");
        String toDate = request.getParameter("toDate");
        String filter = request.getParameter("filter");
        int page = Integer.parseInt(request.getParameter("page"));

        int pageLen = RegistryConstants.ITEMPS_PER_PAGE;
        int start = (pageLen) * (page - 1);

        if (resourcePath != null && resourcePath.equals("")) {
            resourcePath = null;
        }

        if (userName != null && userName.equals("")) {
            userName = null;
        }

        if (fromDate != null && fromDate.equals("")) {
            fromDate = null;
        }

        if (toDate != null && toDate.equals("")) {
            toDate = null;
        }

        if (filter == null) {
            filter = "1";
        }

        int filterValue = -1;
        if (filter.equals("resourceAdd")) {
            filterValue = LogEntry.ADD;

        } else if (filter.equals("resourceUpdate")) {
            filterValue = LogEntry.UPDATE;

        } else if (filter.equals("commentings")) {
            filterValue = LogEntry.COMMENT;

        } else if (filter.equals("taggings")) {
            filterValue = LogEntry.TAG;

        } else if (filter.equals("ratings")) {
            filterValue = LogEntry.RATING;

        } else if (filter.equals("restore")) {
            filterValue = LogEntry.RESTORE;

        } else if (filter.equals("delete")) {
            filterValue = LogEntry.DELETE_RESOURCE;
        }

        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            LogEntry[] logs = userRegistry.getLogs(resourcePath, filterValue,
                    userName, computeDate(fromDate), computeDate(toDate), true);

            activityBean.setActivity(constructActivityStatements(logs));

        } catch (RegistryException e) {

            String msg = "Failed to get activities for generating activity search results " +
                    e.getMessage();
            log.error(msg, e);
            activityBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.ACTIVITY_BEAN, activityBean);
    }

    private static List <String> constructActivityStatements(LogEntry[] logs) {

        List <String> activity = new ArrayList<String>();

        for (int i = 0; i < logs.length; i++) {

            LogEntry logEntry = logs[i];

            if (logEntry.getAction() == LogEntry.ADD) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has added the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.UPDATE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has updated the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.DELETE_RESOURCE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has deleted the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.COMMENT) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has commented on resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> on " + CommonUtil.formatDate(logEntry.getDate()) + " with the following comment.<br/>" + logEntry.getActionData();
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.TAG) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has tagged the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> with tag '" + logEntry.getActionData() + "' on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.RATING) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has rated the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> with rating " + logEntry.getActionData() + " on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);

            } else if (logEntry.getAction() == LogEntry.RESTORE) {
                String entry = "<a href='/wso2registry/system/people/" + logEntry.getUserName() + "'>" + logEntry.getUserName() + "</a>" + " has restored the resource '" + "<a href='/wso2registry/web" + logEntry.getResourcePath() + "'>" + logEntry.getResourcePath() + "</a> to the version " + logEntry.getActionData() + " on " + CommonUtil.formatDate(logEntry.getDate()) + ".";
                activity.add(entry);
            }
        }

        return activity;
    }

    /**
     * Converts given strings to Dates
     *
     * @param dateString Allowed format mm/dd/yyyy
     * @return Date corresponding to the given string date
     */
    private static Date computeDate(String dateString) throws RegistryException {

        if (dateString == null || dateString.length() == 0) {
            return null;
        }

        DateFormat formatter = DateFormat.getDateInstance(DateFormat.SHORT);
        try {
            return formatter.parse(dateString);
        } catch (ParseException e) {
            String msg = "Date format is invalid: " + dateString;
            throw new RegistryException(msg, e);
        }
    }
}
