/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.actions.utils.AdvancedResourceQuery;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.beans.AdvancedSearchResultsBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Collection;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;

import javax.servlet.http.HttpServletRequest;

public class AdvancedSearchResultsBeanPopulator {

    private static final Log log = LogFactory.getLog(AdvancedSearchResultsBeanPopulator.class);

    public static void populate(HttpServletRequest request) {

        AdvancedSearchResultsBean advancedSearchResultsBean = new AdvancedSearchResultsBean();

        String resourceName = request.getParameter("resourcePath");
        String authorName = request.getParameter("author");
        String updaterName = request.getParameter("updater");
        String createdAfter = request.getParameter("createdAfter");
        String createdBefore = request.getParameter("createdBefore");
        String updatedAfter = request.getParameter("updatedAfter");
        String updatedBefore = request.getParameter("updatedBefore");
        String tags = request.getParameter("tags");
        String commentWords = request.getParameter("commentWords");
        String propertyName = request.getParameter("propertyName");
        String propertyValue = request.getParameter("propertyValue");
        String content = request.getParameter("content");

        try {
            UserRegistry registry = CommonUtil.getUserRegistry(request);

            AdvancedResourceQuery query = new AdvancedResourceQuery();
            query.setResourceName(resourceName);
            query.setAuthorName(authorName);
            query.setUpdaterName(updaterName);
            query.setCreatedAfter(CommonUtil.computeDate(createdAfter));
            query.setCreatedBefore(CommonUtil.computeDate(createdBefore));
            query.setUpdatedAfter(CommonUtil.computeDate(updatedAfter));
            query.setUpdatedBefore(CommonUtil.computeDate(updatedBefore));
            query.setCommentWords(commentWords);
            query.setTags(tags);
            query.setPropertyName(propertyName);
            query.setPropertyValue(propertyValue);
            query.setContent(content);

            Resource qResults = query.execute(registry);

            String[] childPaths = (String[]) qResults.getContent();
            for (String childPath : childPaths) {

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(childPath);

                if (childPath != null) {
                    if (RegistryConstants.ROOT_PATH.equals(childPath)) {
                        resourceData.setName("root");
                    } else {
                        String[] parts = childPath.split(RegistryConstants.PATH_SEPARATOR);
                        resourceData.setName(parts[parts.length - 1]);
                    }
                }

                try {
                    Resource child = registry.get(childPath);

                    resourceData.setResourceType(child instanceof Collection ?
                            ResourceData.COLLECTION : ResourceData.RESOURCE);
                    resourceData.setAuthorUserName(child.getAuthorUserName());
                    resourceData.setDescription(child.getDescription());
                    resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                    resourceData.setCreatedOn(child.getCreatedTime());
                    CommonUtil.populateAverageStars(resourceData);

                    child.discard();

                    advancedSearchResultsBean.getResourceDataList().add(resourceData);

                } catch (AuthorizationFailedException e) {
                    log.info(
                            "AuthorizationFailedException inside execute method of AdvancedSearchAction");
                    // do not show unauthorized resource in search results.
                }

            }

        } catch (RegistryException e) {

            String msg = "Failed to get advanced search results. " + e.getMessage();
            log.error(msg, e);
            advancedSearchResultsBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(
                UIConstants.ADVANCED_SEARCH_RESULTS_BEAN, advancedSearchResultsBean);
    }
}
