/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.*;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.beans.AssociationTreeBean;
import org.wso2.registry.web.utils.CommonUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Arrays;

public class AssociationTreeBeanPopulator {

    private static final Log log = LogFactory.getLog(AssociationTreeBeanPopulator.class);

    public static void populate(HttpServletRequest request) {

        AssociationTreeBean associationTreeBean = new AssociationTreeBean();

        String resourcePath = request.getParameter("resourcePath");
        String associationType = request.getParameter("type");

        associationTreeBean.setResourcePath(resourcePath);
        associationTreeBean.setAssoType(associationType);
        associationTreeBean.setAssoIndex(0);

        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);

            List fatherAsso = provideChildAssociations(resourcePath, associationType, userRegistry);
            Iterator ifatherAssofaterAsso = fatherAsso.iterator();

            while (ifatherAssofaterAsso.hasNext() && fatherAsso != null) {
                Association asso = (Association) ifatherAssofaterAsso.next();
                createAssociationTree(
                        asso.getDestinationPath(), associationTreeBean, userRegistry);
            }

        } catch (RegistryException e) {
            String msg = "Failed to generate association tree of the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            associationTreeBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.ASSOCIATION_TREE_BEAN, associationTreeBean);
    }

    private static void createAssociationTree(String path, AssociationTreeBean associationTreeBean, UserRegistry registry) throws RegistryException {

        associationTreeBean.setAssoIndex(associationTreeBean.getAssoIndex() + 1);
        
        Resource father = registry.get(path);

        String fatherResourceType = father instanceof Collection ? ResourceData.COLLECTION : ResourceData.RESOURCE;
        List fatherAsso = provideChildAssociations(path, associationTreeBean.getAssoType(), registry);
        associationTreeBean.getTreeCache().add(path);

        boolean foundInfinite = false;

        if (fatherAsso == null) foundInfinite = true;

        String associationTreePart = "<div class=\"father-object\" >";
        associationTreePart += "<ul class=\"tree-row-object\"><li class=\"first\">";
        if (!foundInfinite) {
            if (!fatherAsso.isEmpty()) {
                associationTreePart += "<a onclick=\"showHideCommon('y_" + associationTreeBean.getAssoIndex() + "');showHideCommon('xminus_" + associationTreeBean.getAssoIndex() + "');showHideCommon('xplus_" + associationTreeBean.getAssoIndex() + "');\">" +
                        "<img src=\"/wso2registry/admin/images/icon-tree-minus.jpg\" id=\"xminus_" + associationTreeBean.getAssoIndex() + "\" style=\"margin-right:2px;\" />" +
                        "<img src=\"/wso2registry/admin/images/icon-tree-plus.jpg\" id=\"xplus_" + associationTreeBean.getAssoIndex() + "\" style=\"margin-right:2px;display:none;\" />" +
                        "</a>";
            } else {
                associationTreePart += "<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:15px;\" />";
            }
        } else {
            associationTreePart += "<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:15px;\" />";
        }
        if (fatherResourceType.equals(ResourceData.COLLECTION)) {
            associationTreePart += "<img src=\"/wso2registry/admin/images/icon-folder-small.gif\" style=\"margin-right:2px;\" />";
        } else {
            associationTreePart += "<img src=\"/wso2registry/admin/images/editshred.png\" style=\"margin-right:2px;\" />";
        }

        associationTreePart += "<a href=\"" + calculateRelativePath(path) + "\">";
        if (foundInfinite) {
            associationTreePart += path + "(infinite loop..)";
        } else {
            associationTreePart += path;
        }
        associationTreePart += "</a></li>";
        if (!associationTreeBean.getAssoType().equals(UIConstants.ASSOCIATION_TYPE01))
            associationTreePart += "<li class=\"second\">" + associationTreeBean.getAssoType() + "</li>";
        associationTreePart += "</ul></div>";
        if (!foundInfinite && !fatherAsso.isEmpty()) {
            associationTreePart += "<div class=\"child-objects\" id=\"y_" + associationTreeBean.getAssoIndex() + "\">";
            Iterator iFaterAsso = fatherAsso.iterator();
            while (iFaterAsso.hasNext()) {
                Association asso = (Association) iFaterAsso.next();

                // if path is equal to the destination path, it is a backward association. we are only displaying
                // the forward associations here
                if (!path.equals(asso.getDestinationPath())) {
                    if(!associationTreeBean.getTreeCache().contains(asso.getDestinationPath())){
                        createAssociationTree(asso.getDestinationPath(), associationTreeBean, registry);
                    }
                }
            }
            associationTreePart += "</div>";

        }

        father.discard();

        associationTreeBean.setAssociationTree(associationTreeBean.getAssociationTree() + associationTreePart);
    }

    private static String calculateRelativePath(String resourcePath) {
        if (resourcePath.startsWith("http://")) {
            return resourcePath;
        }

        String relativePath = resourcePath;

        if (RegistryConstants.ROOT_PATH.equals(resourcePath)) {
            relativePath = "";
        } else {
            if (resourcePath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                relativePath = resourcePath.substring(1, resourcePath.length());
            }
        }
        return relativePath;
    }

    private static List provideChildAssociations(
            String resourcePath, String assoType, UserRegistry registry)
            throws RegistryException {

        List tmpAssociations = new ArrayList();
        List associations = new ArrayList();
        Resource resource = registry.get(resourcePath);
        Association[] asso = registry.getAllAssociations(resource.getPath());

        tmpAssociations.addAll(Arrays.asList(asso));

        Iterator iAssociations = tmpAssociations.iterator();
        while (iAssociations.hasNext()) {
            Association tmpAsso = (Association) iAssociations.next();
            //Get all associations filtered by it's association type
            if (!resourcePath.equals(tmpAsso.getDestinationPath())) {
                if (tmpAsso.getAssociationType().equals(UIConstants.ASSOCIATION_TYPE01) && assoType.equals(UIConstants.ASSOCIATION_TYPE01))
                    associations.add(tmpAsso);
                if (!tmpAsso.getAssociationType().equals(UIConstants.ASSOCIATION_TYPE01) && !assoType.equals(UIConstants.ASSOCIATION_TYPE01))
                    associations.add(tmpAsso);
            }
        }

        resource.discard();
        // if path is equal to the destination path, it is a backward association. we are only displaying
        // the forward associations here

        return associations;
    }
}
