/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.Comment;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.beans.CommentBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

public class CommentBeanPopulator {

    private static final Log log = LogFactory.getLog(CommentBeanPopulator.class);

    public static void populate(
            HttpServletRequest request, UserRegistry userRegistry, Resource resource) {

        CommentBean commentBean = new CommentBean();

        try {
            Comment[] c = userRegistry.getComments(resource.getPath());
            List <Comment> comments = new ArrayList <Comment> ();
            comments.addAll(Arrays.asList(c));
            commentBean.setComments(comments);

        } catch (RegistryException e) {

            String msg = "Failed to get comment information of the resource " +
                    resource.getPath() + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            commentBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.COMMENT_SECTION_BEAN, commentBean);
    }
}
