/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.beans.CommonBean;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserStoreException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;

public class CommonBeanPopulator {

    private static final Log log = LogFactory.getLog(CommonBeanPopulator.class);

    public static void populate(HttpServletRequest request, ResourcePath path) {

        if (path == null) {
            path = (ResourcePath) request.getSession().getAttribute(UIConstants.PROCESSED_PATH);
        }

        CommonBean commonBean = new CommonBean();
        commonBean.setPath(path.getCompletePath());

        String versionPath;
        if (path.parameterExists("version")) {
            versionPath = path.getPath() + RegistryConstants.URL_SEPARATOR + "version" +
                    RegistryConstants.URL_PARAMETER_SEPARATOR + path.getParameterValue("version");
        } else {
            versionPath = path.getPath();
        }
        commonBean.setPathWithVersion(versionPath);

        UserRegistry userRegistry;
        try {
            userRegistry = CommonUtil.getUserRegistry(request);
        } catch (RegistryException e) {
            String msg = "Failed to get user's registry instance. " + e.getMessage();
            log.error(msg, e);
            commonBean.setErrorMessage(msg);
            request.getSession().setAttribute(UIConstants.COMMON_BEAN, commonBean);
            return;
        }

        String userName = userRegistry.getUserName();
        commonBean.setUserName(userName);
        commonBean.setLoggedIn(!RegistryConstants.ANONYMOUS_USER.equals(userName));

        if (path.parameterExists("version")) {
            commonBean.setVersionView(true);
        } else {
            commonBean.setVersionView(false);
        }

        try {
            Authorizer authorizer = userRegistry.getUserRealm().getAuthorizer();
            boolean putAllowed = authorizer.
                    isUserAuthorized(userName, path.getPath(), ActionConstants.PUT);
            commonBean.setPutAllowed(putAllowed);

        } catch (UserStoreException e) {

            String msg = "Failed to check put permissions of user " + userName +
                    " on the resource " + path + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            commonBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.COMMON_BEAN, commonBean);
    }
}
