/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.web.beans.ContentBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.MediaTypesReader;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.Resource;
import org.wso2.registry.Collection;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

public class ContentBeanPopulator {

    private static final Log log = LogFactory.getLog(ContentBeanPopulator.class);

    public static void populate(HttpServletRequest request, Resource resource, ResourcePath path) {

        ContentBean contentBean = new ContentBean();

        contentBean.setCollection(resource instanceof Collection);

        List <String> browseViewData = resource.getPropertyValues("registry.browse-views");
        Map browseViews = new HashMap();
        if (browseViewData != null) {
            Iterator <String> iBrowseData = browseViewData.iterator();
            while (iBrowseData.hasNext()) {
                String browseView = iBrowseData.next();
                if (browseView.indexOf(":") == -1) {
                    browseViews.put(browseView, browseView);
                } else {
                    String[] parts = browseView.split(":");
                    browseViews.put(parts[0], parts[1]);
                }
            }
        }
        contentBean.setBrowseViews(browseViews);

        List <String> editViewdata = resource.getPropertyValues("registry.edit-views");
        Map editViews = new HashMap();
        if (editViewdata != null) {
            Iterator <String> iEditData = editViewdata.iterator();
            while (iEditData.hasNext()) {
                String editView = iEditData.next();
                if (editView.indexOf(":") == -1) {
                    editViews.put(editView, editView);
                } else {
                    String[] parts = editView.split(":");
                    editViews.put(parts[0], parts[1]);
                }
            }
        }
        contentBean.setEditViews(editViews);

        String renderingMethod =
                resource.getProperty(RegistryConstants.UI_RENDERING_METHOD_PROPERTY);
        contentBean.setRenderingMethod(renderingMethod);


        try {
            if (renderingMethod != null && !RegistryConstants.R_RAW.equals(renderingMethod)) {
                String contentString = (String) resource.getContent();
                contentBean.setContentString(contentString);
            }

        } catch (RegistryException e) {
            String msg = "Failed to get content of the resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            CommonUtil.addErrorMessage(request, msg);
            contentBean.setErrorMessage(msg);
        }

        try {
            contentBean.setCollectionTypes(MediaTypesReader.getCollectionMediaTypes(request));
        } catch (RegistryException e) {
            String msg = "Failed to get defined collection media types. " + e.getMessage();
            log.error(msg, e);

            contentBean.setErrorMessage(msg);
            CommonUtil.addErrorMessage(request, msg);
        }

        contentBean.setContentPath(path.getCompletePath());
        contentBean.setResource(resource);

        request.getSession().setAttribute(UIConstants.RESOURCE_CONTENT_BEAN, contentBean);
    }
}
