/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.beans.DependenciesBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.Association;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

public class DependenciesBeanPopulator {

    private static final Log log = LogFactory.getLog(DependenciesBeanPopulator.class);

    public static void populate(
            HttpServletRequest request, UserRegistry userRegistry, ResourcePath resourcePath) {

        DependenciesBean dependenciesBean = new DependenciesBean();

        try {
            Association[] asso = userRegistry.getAllAssociations(resourcePath.getPath());
            List <Association> associations = new ArrayList <Association> ();
            associations.addAll(Arrays.asList(asso));
            dependenciesBean.setAssociations(associations);

        } catch (RegistryException e) {

            String msg = "Failed to get dependencies of resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            dependenciesBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.DEPENDENCIES_BEAN, dependenciesBean);
    }
}
