/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.web.beans.LifeCycleBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;
import java.util.HashMap;

public class LifeCycleBeanPopulator {

    private static final Log log = LogFactory.getLog(LifeCycleBeanPopulator.class);

    public static void populate(
            HttpServletRequest request,
            UserRegistry registry,
            Resource resource,
            ResourcePath resourcePath) {

        LifeCycleBean lifeCycleBean = new LifeCycleBean();

        try {
            Map <String, String[]> availableActions = new HashMap <String, String[]> ();
            List<String> aspects = resource.getAspects();
            if (aspects != null) {
                for (String aspect : aspects) {
                    String[] actions = registry.getAspectActions(resourcePath.getPath(), aspect);
                    if (actions == null) continue;
                    availableActions.put(aspect, actions);
                }
            }
            
            lifeCycleBean.setAvailableActions(availableActions);
            lifeCycleBean.setAspectsToAdd(registry.getAvailableAspects());

        } catch (RegistryException e) {

            String msg = "Failed to get life cycle information of resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            lifeCycleBean.setErrorMessage(msg);
        }


        request.getSession().setAttribute(UIConstants.LIFE_CYCLE_BEAN, lifeCycleBean);
    }
}
