/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.beans.PeopleBean;
import org.wso2.registry.web.UIConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;

public class PeopleBeanPopulator {

    private static final Log log = LogFactory.getLog(PeopleBeanPopulator.class);

    public static void populate(HttpServletRequest request) {

        PeopleBean peopleBean = new PeopleBean();

        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            UserRealm userRealm = userRegistry.getUserRealm();
            String userName = userRegistry.getUserName();

            peopleBean.setAddAllowed(userRealm.getAuthorizer().isUserAuthorized(
                    userName, AccessControlConstants.USER_RESOURCE, AccessControlConstants.ADD));

            peopleBean.setDeleteAllowed(userRealm.getAuthorizer().isUserAuthorized(
                    userName, AccessControlConstants.USER_RESOURCE, AccessControlConstants.DELETE));

            String[] userNamesArray = userRealm.getUserStoreReader().getAllUserNames();
            for (int i = 0; i < userNamesArray.length; i++) {
                String friendlyName = (String) userRealm.getUserStoreAdmin().
                        getUserProperties(userNamesArray[i]).get("friendlyName");
                if (friendlyName == null) {
                    friendlyName = "";
                }

                peopleBean.getUserNames().put(userNamesArray[i], friendlyName);
            }

            String[] roleNamesArray = userRealm.getUserStoreReader().getAllRoleNames();
            for (int i = 0; i < roleNamesArray.length; i++) {
                peopleBean.getRoleNames().add(roleNamesArray[i]);
            }

        } catch (Exception e) {
            String msg = "Failed to get information about registered users. Caused by: "
                    + e.getMessage();
            log.error(msg, e);
            peopleBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.PEOPLE_BEAN, peopleBean);
    }
}
