/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.web.beans.ResourceMetadataBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.utils.WebResourcePath;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

public class ResourceMetadataPopulator {

    private static final Log log = LogFactory.getLog(ResourceMetadataPopulator.class);

    public static void populate(HttpServletRequest request, Resource resource) {

        ResourcePath path = (ResourcePath) request.
                getSession().getAttribute(UIConstants.PROCESSED_PATH);

        ResourceMetadataBean bean = new ResourceMetadataBean();

        bean.setAuthor(resource.getAuthorUserName());
        bean.setCollection(resource instanceof Collection);
        bean.setDescription(resource.getDescription());
        bean.setLastUpdater(resource.getLastUpdaterUserName());
        bean.setMediaType(resource.getMediaType());
        bean.setPath(resource.getPath());

        bean.setFormattedCreatedOn(CommonUtil.formatDate(resource.getCreatedTime()));
        bean.setFormattedLastModified(CommonUtil.formatDate(resource.getLastModified()));

        bean.setActiveResourcePath(path.getPath());

        if (path.parameterExists("version")) {
            bean.setVersionView(true);
            bean.setPermalink(path.getCompletePath());
        } else {
            bean.setVersionView(false);
            bean.setPermalink(resource.getPermanentPath());
        }

        bean.setNavigatablePaths(constructNavigatablePaths(path.getCompletePath()));

        bean.setContentPath(path.getCompletePath());

        String reqURL = request.getRequestURL().toString();
        bean.setServerBaseURL(reqURL.substring(0, reqURL.indexOf("/wso2registry")));

        request.getSession().setAttribute(UIConstants.RESOURCE_METADATA_BEAN, bean);
    }

    private static List <WebResourcePath> constructNavigatablePaths(String rawPath) {

        List <WebResourcePath> navigatablePaths = new ArrayList <WebResourcePath> ();

        if (rawPath.equals(RegistryConstants.ROOT_PATH)) {

            WebResourcePath webPath = new WebResourcePath();
            webPath.setNavigateName("/");
            webPath.setNavigatePath("#");

            navigatablePaths.add(webPath);

        } else {

            // first add the root path
            WebResourcePath rootPath = new WebResourcePath();
            rootPath.setNavigateName("/");
            rootPath.setNavigatePath(RegistryConstants.ROOT_PATH);
            navigatablePaths.add(rootPath);

            String preparedPath = rawPath;
            if (preparedPath.startsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.substring(RegistryConstants.PATH_SEPARATOR.length());
            }

            if (preparedPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                preparedPath = preparedPath.
                        substring(0, preparedPath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            String[] parts = preparedPath.split(RegistryConstants.PATH_SEPARATOR);
            for (int i = 0; i < parts.length; i++) {
                WebResourcePath resourcePath = new WebResourcePath();

                if (i == parts.length - 1) {
                    String[] q = parts[i].split(RegistryConstants.URL_SEPARATOR);
                    if (q.length == 2) {
                        if (q[1].startsWith("version:")) {
                            String versionNumber = q[1].substring("version:".length());
                            String navName = q[0] + " (version " + versionNumber + ")";
                            resourcePath.setNavigateName(navName);
                        } else {
                            resourcePath.setNavigateName(parts[i]);
                        }
                    } else {
                        resourcePath.setNavigateName(parts[i]);
                    }
                } else {
                    resourcePath.setNavigateName(parts[i]);
                }

                String tempPath = "";
                for (int j = 0; j < i + 1; j++) {
                    tempPath = tempPath + RegistryConstants.PATH_SEPARATOR + parts[j];
                }
                resourcePath.setNavigatePath(tempPath);
                navigatablePaths.add(resourcePath);
            }
        }

        return navigatablePaths;
    }
}
