/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.*;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.ResourceData;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.beans.SearchResultsBean;
import org.wso2.registry.web.UIConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.ArrayList;

public class SearchResultsBeanPopulator {

    private static final Log log = LogFactory.getLog(SearchResultsBeanPopulator.class);

    public static void populate(HttpServletRequest request) {

        SearchResultsBean searchResultsBean = new SearchResultsBean();

        try {
            UserRegistry registry = CommonUtil.getUserRegistry(request);

            if (request.getParameter("searchType").equalsIgnoreCase("Tag")) {
                String tag = request.getParameter("criteria");
                searchResultsBean.setResourceDataList(searchByTags(tag, registry));
            } else {
                String content = request.getParameter("criteria");
                searchResultsBean.setResourceDataList(searchByContent(content, registry));
            }

        } catch (RegistryException e) {

            String msg = "Failed to generate search results. " + e.getMessage();
            log.error(msg, e);
            searchResultsBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.SEARCH_RESULTS_BEAN, searchResultsBean);
    }

    private static List <ResourceData> searchByTags(String tag, UserRegistry registry)
            throws RegistryException {

        List <ResourceData> resourceDataList = new ArrayList <ResourceData> ();

        if (tag != null && tag.length() != 0) {
            TaggedResourcePath[] taggedPaths = registry.getResourcePathsWithTag(tag);
            for (TaggedResourcePath taggedPath : taggedPaths) {
                String resultPath = taggedPath.getResourcePath();

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(resultPath);

                if (resultPath != null) {
                    if (resultPath.equals(RegistryConstants.ROOT_PATH)) {
                        resourceData.setName(RegistryConstants.ROOT_PATH);
                    } else {
                        String[] parts = resultPath.split(RegistryConstants.PATH_SEPARATOR);
                        resourceData.setName(parts[parts.length - 1]);
                    }
                }

                try {
                    Resource child = registry.get(resultPath);

                    resourceData.setResourceType(child instanceof Collection ?
                            ResourceData.COLLECTION : ResourceData.RESOURCE);
                    resourceData.setAuthorUserName(child.getAuthorUserName());
                    resourceData.setDescription(child.getDescription());
                    resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                    resourceData.setCreatedOn(child.getCreatedTime());
                    resourceData.setTagCounts(taggedPath.getTagCounts());
                    CommonUtil.populateAverageStars(resourceData);

                    child.discard();

                    resourceDataList.add(resourceData);

                } catch (AuthorizationFailedException e) {
                    // do not show unauthorized resources in search results
                }
            }
        }

        return resourceDataList;
    }

    private static List <ResourceData> searchByContent(String content, UserRegistry registry)
            throws RegistryException {

        List <ResourceData> resourceDataList = new ArrayList <ResourceData> ();

        if(content != null && content.length() != 0){
            String [] paths = (String [])registry.searchContent(content).getContent();
            for(int i=0; i<paths.length; i++){
                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(paths[i]);

                if (paths[i] != null) {
                    if (paths[i].equals(RegistryConstants.ROOT_PATH)) {
                        resourceData.setName(RegistryConstants.ROOT_PATH);
                    } else {
                        String[] parts = paths[i].split(RegistryConstants.PATH_SEPARATOR);
                        resourceData.setName(parts[parts.length - 1]);
                    }
                }

                try {
                    Resource child = registry.get(paths[i]);

                    resourceData.setResourceType(child instanceof Collection ?
                            ResourceData.COLLECTION : ResourceData.RESOURCE);
                    resourceData.setAuthorUserName(child.getAuthorUserName());
                    resourceData.setDescription(child.getDescription());
                    resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                    resourceData.setCreatedOn(child.getCreatedTime());
                    CommonUtil.populateAverageStars(resourceData);

                    child.discard();

                    resourceDataList.add(resourceData);

                } catch (AuthorizationFailedException e) {
                    // do not show unauthorized resources in search results
                }
            }
        }

        return resourceDataList;
    }
}
