/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.Tag;
import org.wso2.registry.Resource;
import org.wso2.registry.web.beans.TagBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.List;
import java.util.ArrayList;

public class TagBeanPopulator {

    private static final Log log = LogFactory.getLog(TagBeanPopulator.class);

    public static void populate(
            HttpServletRequest request, UserRegistry userRegistry, Resource resource) {

        TagBean tagBean = new TagBean();

        try {
            Tag[] t = userRegistry.getTags(resource.getPath());
            List <Tag> tags = new ArrayList <Tag> ();
            tags.addAll(Arrays.asList(t));
            tagBean.setTags(tags);

        } catch (RegistryException e) {

            String msg = "Failed to get tagging information of resource " + resource.getPath() + ". " + e.getMessage();
            log.error(msg, e);

            CommonUtil.addErrorMessage(request, msg);
            tagBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.TAG_SECTION_BEAN, tagBean);
    }
}
