/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.LogEntry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.Authorizer;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.beans.UserBean;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

public class UserBeanPopulator {

    private static final Log log = LogFactory.getLog(UserBeanPopulator.class);

    public static void populate(HttpServletRequest request, String displayUserName) {

        UserBean userBean = new UserBean();
        userBean.setDisplayUserName(displayUserName);

        try {

            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            UserRealm realm = userRegistry.getUserRealm();
            String userName = userRegistry.getUserName();

            if (userName.equals(displayUserName) &&
                    !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {
                userBean.setUserEditable(true);
            }

            Authorizer auth = realm.getAuthorizer();
            if (auth.isUserAuthorized(
                    userName, AccessControlConstants.USER_RESOURCE, AccessControlConstants.EDIT) &&
                    !RegistryConstants.SYSTEM_USER.equals(displayUserName) &&
                    !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {

                userBean.setUserEditable(true);
            }

            if (auth.isUserAuthorized(userName,
                    AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.ADD) &&
                    auth.isUserAuthorized(userName,
                            AccessControlConstants.ROLE_RESOURCE, AccessControlConstants.DELETE) &&
                    !RegistryConstants.SYSTEM_USER.equals(displayUserName) &&
                    !RegistryConstants.ANONYMOUS_USER.equals(displayUserName)) {

                userBean.setUserAdmin(true);
            }

            userBean.setFriendlyName((String) realm.
                    getUserStoreAdmin().getUserProperties(displayUserName).get("friendlyName"));
            if (userBean.getFriendlyName() == null) {
                userBean.setFriendlyName("");
            }

            String[] rolesArray = realm.getUserStoreReader().getUserRoles(displayUserName);
            userBean.getUserRoles().addAll(Arrays.asList(rolesArray));

            String[] roleNamesArray = realm.getUserStoreReader().getAllRoleNames();
            userBean.getAllRoles().addAll(Arrays.asList(roleNamesArray));

            LogEntry[] userLogs = userRegistry.getLogs(null, -1, displayUserName, null, null, true);
            for (int i = 0; i < userLogs.length && i < 10; i++) {
                LogEntry logEntry = userLogs[i];
                userBean.getUserActivity().add(logEntry.getText());
            }

        } catch (Exception e) {
            String msg = "Failed to get user information of the user " +
                    displayUserName + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            userBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.USER_BEAN, userBean);
    }
}
