/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.populators;

import org.wso2.registry.web.beans.VersionsBean;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.actions.utils.VersionPath;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletRequest;

public class VersionsBeanPopulator {

    private static final Log log = LogFactory.getLog(VersionsBeanPopulator.class);

    public static void populate(HttpServletRequest request, String path) {

        VersionsBean versionsBean = new VersionsBean();

        versionsBean.setResourcePath(path);

        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            Resource currentResource = userRegistry.get(path);
            versionsBean.setPermalink(currentResource.getPermanentPath());
            currentResource.discard();

            String[] versions = userRegistry.getVersions(path);
            for (String version : versions) {
                VersionPath versionPath = new VersionPath();
                versionPath.setCompleteVersionPath(version);
                versionPath.setActiveResourcePath(path);

                ResourcePath resourcePath = new ResourcePath(version);
                if (!resourcePath.isCurrentVersion()) {
                    long versionNumber = Long.parseLong(resourcePath.
                            getParameterValue(RegistryConstants.VERSION_PARAMETER_NAME));
                    versionPath.setVersionNumber(versionNumber);
                }

                Resource versionResource = userRegistry.get(version);
                if (versionResource != null) {
                    versionPath.setUpdater(versionResource.getLastUpdaterUserName());
                    versionPath.setUpdatedOn(versionResource.getLastModified());
                }

                versionsBean.getVersionPaths().add(versionPath);
                versionResource.discard();
            }

        } catch (RegistryException e) {

            String msg = "Failed to get version information of resource " +
                    path + ". " + e.getMessage();
            log.error(msg, e);
            versionsBean.setErrorMessage(msg);
        }

        request.getSession().setAttribute(UIConstants.VERSIONS_BEAN, versionsBean);
    }
}
