/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.CollectionImpl;
import org.wso2.registry.Registry;
import org.wso2.registry.exceptions.RegistryException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class AddCollectionProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String parentPath = request.getParameter("parentPath");
        String collectionName = request.getParameter("collectionName");
        String mediaType = request.getParameter("mediaType");
        String description = request.getParameter("description");
        String redirectPath = parentPath;

        String path;
        if (parentPath.equals(RegistryConstants.ROOT_PATH)) {
            path = RegistryConstants.ROOT_PATH + collectionName;
        } else {
            path = parentPath + RegistryConstants.PATH_SEPARATOR + collectionName;
        }

        CollectionImpl collection = new CollectionImpl();
        collection.setPath(path);
        if (!mediaType.equals(RegistryConstants.DEFAULT_MEDIA_TYPE)) {
            collection.setMediaType(mediaType);
        }
        collection.setDescription(description);

        try {
            Registry registry = getUserRegistry(request);
            registry.put(path, collection);
        } catch (RegistryException e) {
            String msg = "Failed to add new collection " + collectionName + ". " + e.getMessage();
            log.error(msg, e);
            addErrorMessage(request, msg);
        }

        redirect(response, "/wso2registry/web" + redirectPath);
    }
}
