/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.populators.DependenciesBeanPopulator;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.Association;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;

public class AddRemoveAssociationProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String associationType = request.getParameter("type");
        String associationPaths = request.getParameter("associationPaths");
        String todo = request.getParameter("todo");

        try {

            UserRegistry registry = getUserRegistry(request);

            if(todo.equals("add")){
                registry.addAssociation(resourcePath, associationPaths, associationType);
            } else {
                registry.removeAssociation(resourcePath, associationPaths, associationType);
            }

            Association[] deps = registry.getAllAssociations(resourcePath);
            List assoList = new ArrayList();
            assoList.addAll(Arrays.asList(deps));

            ResourcePath path = new ResourcePath(resourcePath);
            DependenciesBeanPopulator.populate(request, registry, path);

            if(associationType.equals(UIConstants.ASSOCIATION_TYPE01)) {
                request.getSession().setAttribute("type","dep");
            } else{
                request.getSession().setAttribute("type","asso");
            }

            forward(request, response, UIConstants.AJAX_ASSOCIATIONS_JSP);

        } catch (RegistryException e) {

            String msg = "Failed to update association. Resource path: " + resourcePath +
                    ", Association path: " + associationPaths + ", Association type: " +
                    associationType + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
        }
    }
}
