/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class AddTextResourceProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String parentPath = request.getParameter("path");
        String resourceName = request.getParameter("filename");

        String resourcePath;
        if (RegistryConstants.ROOT_PATH.equals(parentPath)) {
            resourcePath = RegistryConstants.ROOT_PATH + resourceName;
        } else {
            resourcePath = parentPath + RegistryConstants.PATH_SEPARATOR + resourceName;
        }

        String mediaType = request.getParameter("mediaType");
        String description = request.getParameter("description");
        String contentString = request.getParameter("content");
        byte[] content = null;
        if (contentString != null) {
            content = contentString.getBytes();
        }

        try {
            UserRegistry userRegistry = getUserRegistry(request);
            Resource resource = userRegistry.newResource();

            resource.setMediaType(mediaType);
            resource.setDescription(description);
            resource.setContent(content);

            userRegistry.put(resourcePath, resource);
            resource.discard();

        } catch (RegistryException e) {
            String msg = "Failed to add resource with text based content to path " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            addErrorMessage(request, msg);
        }

        redirect(response, "/wso2registry/web" + parentPath);
    }
}
