/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

public class AddUserProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String newUserName = request.getParameter("newUserName");
        String password = request.getParameter("password");
        String confirmedPassword = request.getParameter("confirmedPassword");
        String roleName = request.getParameter("roleName");
        String friendlyName = request.getParameter("friendlyName");

        if (newUserName == null || password == null || confirmedPassword == null) {
            String msg = "Failed to add the new user " +
                    newUserName + ". Required parameters are missing";
            log.error(msg);
            sendErrorContent(response, msg);
            return;
        }

        if (!password.equals(confirmedPassword)) {
            String msg = "Failed to add the new user " + newUserName +
                    ". Confirmed passward does not match with the password.";
            log.error(msg);
            sendErrorContent(response, msg);
            return;
        }

        try {
            UserRegistry userRegistry = getUserRegistry(request);
            UserRealm userRealm = userRegistry.getUserRealm();

            userRealm.getUserStoreAdmin().addUser(newUserName, password);

            Map currentProperties = userRealm.getUserStoreAdmin().getUserProperties(newUserName);
            currentProperties.put("friendlyName", friendlyName);
            userRealm.getUserStoreAdmin().setUserProperties(newUserName, currentProperties);

            if (roleName != null && !roleName.equalsIgnoreCase("1")) {
                userRealm.getUserStoreAdmin().addUserToRole(newUserName, roleName);
            }
        } catch (Exception e) {

            String msg = "Failed to add the new user " + newUserName + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
            return;
        }

        redirect(response, "/wso2registry/people");
    }
}
