/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Tag;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;

public class ApplyTagProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        try {
            applyTag(request);
        } catch (RegistryException e) {
            try {
                sendTagCloud(request, response, e.getMessage());
                return;
            } catch (RegistryException e1) {
                sendErrorContent(response, e.getMessage() + " | " + e1.getMessage());
            }
        }

        try {
            sendTagCloud(request, response, null);
        } catch (RegistryException e) {
            sendErrorContent(response, e.getMessage());
        }
    }

    private void applyTag(HttpServletRequest request) throws RegistryException {

        UserRegistry userRegistry = CommonUtil.getUserRegistry(request);

        String resourcePath = request.getParameter("resourcePath");
        String tag = request.getParameter("tag");

        userRegistry.applyTag(resourcePath, tag);
    }

    private void sendTagCloud(HttpServletRequest request, HttpServletResponse response, String message)
            throws RegistryException {

        UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
        response.setContentType("text/html");
        PrintWriter out;
        try {
            out = response.getWriter();
        } catch (IOException e) {
            throw new RegistryException(e.getMessage());
        }

        String resourcePath = request.getParameter("resourcePath");
        Tag[] tags = userRegistry.getTags(resourcePath);

        if (message != null && message.length() > 0) {
            out.println(message + "<br/>");
        }

        for (int i = 0; i < tags.length; i++) {
            Tag tag = tags[i];
            out.println("<a href='/wso2registry/search?criteria=" + tag.getTagName() + "&searchType=tag' class='cloud-x" + tag.getCategory() + "'>" + tag.getTagName() + "</a>");
        }

        out.flush();
        out.close();
    }
}
