/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.populators.CommonBeanPopulator;
import org.wso2.registry.web.populators.ContentBeanPopulator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class CopyResourceProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String parentPath = request.getParameter("parentPath");
        String wantedPage = request.getParameter("wantedPage");
        String resourceName = request.getParameter("resourceName");
        if (!parentPath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
            parentPath = parentPath + RegistryConstants.PATH_SEPARATOR;
        }

        String oldResourcePath = request.getParameter("oldResourcePath");
        String destinationPath = request.getParameter("destinationPath");

        String newResourcePath=destinationPath + "/" +resourceName;

        UserRegistry registry;
        try {
            registry = getUserRegistry(request);
            registry.copy(oldResourcePath, newResourcePath);

        } catch (RegistryException e) {
            String msg = "Failed to copy the resource: " + oldResourcePath + " to Destination: " +
                    newResourcePath + ". Caused by: " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
            return;
        }

        ResourcePath parentResourcePath = new ResourcePath(parentPath);
        CommonBeanPopulator.populate(request, parentResourcePath);

        Resource resource;
        try {
            resource = registry.get(parentPath);
        } catch (RegistryException e) {
            String msg = "Failed to get parent collection of resource " +
                    newResourcePath + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
            return;
        }
        ContentBeanPopulator.populate(request, resource, parentResourcePath);
        resource.discard();

        request.getSession().setAttribute(UIConstants.AJAX_GIVEPAGE_LIST, wantedPage);

        forward(request, response,  UIConstants.AJAX_ENTRY_LIST_JSP);
    }
}
