/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class CreateVersionProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");

        try {
            UserRegistry userRegistry = getUserRegistry(request);

            userRegistry.createVersion(resourcePath);

            Resource resource = userRegistry.get(resourcePath);
            String permalink = resource.getPermanentPath();

            String responseHTML = "<a href=\"/wso2registry/web" + permalink + "\">" +
                    CommonUtil.getServerBaseURL(request) + "/wso2registry/web" + permalink;
            sendContent(response, responseHTML);

            resource.discard();

        } catch (RegistryException e) {
            String errorMessage = "Failed to create a version of resource " + resourcePath;
            log.error(errorMessage, e);
            sendContent(response, errorMessage);
        }
    }
}
