/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.Registry;
import org.wso2.registry.utils.RegistryUtils;
import org.wso2.registry.exceptions.RegistryException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class DeleteResourceProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String pathToDelete = request.getParameter("resourcePath");
        String parentPath = RegistryUtils.getParentPath(pathToDelete);

        if (pathToDelete == null || pathToDelete.length() == 0) {
            String msg = "Failed to delete resource. Resource path cannot be empty.";
            log.error(msg);
            addErrorMessage(request, msg);
            redirect(response, "/wso2registry/web" + parentPath);
            return;
        }

        if (pathToDelete.equals(RegistryConstants.ROOT_PATH)) {
            String msg = "Root collection cannot be deleted.";
            log.error(msg);
            addErrorMessage(request, msg);
            redirect(response, "/wso2registry/web" + parentPath);
            return;
        }

        try {
            Registry registry = getUserRegistry(request);
            registry.delete(pathToDelete);
        } catch (RegistryException e) {

            String msg = "Failed to delete the resource " + pathToDelete + ". " + e.getMessage();
            log.error(msg, e);
            addErrorMessage(request, msg);
        }

        redirect(response, "/wso2registry/web" + parentPath);
    }
}
