/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.populators.RatingsBeanPopulator;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.ResourcePath;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class GetRatingProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String rating = request.getParameter("userRating");
        int iRating = Integer.parseInt(rating);

        UserRegistry registry;
        try {
            registry = getUserRegistry(request);
            registry.rateResource(resourcePath, iRating);
        } catch (RegistryException e) {
            String msg = "Failed to rate the resource " + resourcePath +
                    " with rating " + iRating + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
            return;
        }

        RatingsBeanPopulator.populate(request, registry, new ResourcePath(resourcePath));
        forward(request, response, UIConstants.AJAX_RATING_JSP);
    }
}
