/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.Collection;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.actions.utils.ResourceData;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.ArrayList;

public class GetResourceTreeProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String path;
        List resourceDataList = new ArrayList();
        String parentPath;
        String status;
        String errorMessage;
        List navigatablePaths = new ArrayList();
        List collectionTypes = new ArrayList();
        String resourceTree = "";
        int resourceTreeIndex = 1;

        String resourcePath = RegistryConstants.ROOT_PATH;

        try {
            UserRegistry registry = getUserRegistry(request);
            ResourceTreeData resourceTreeData = new ResourceTreeData();
            fillResourceTree(resourcePath, resourceTreeData, registry);

            String textBoxId = request.getParameter("textBoxId");

            String displayHTML = "<h1 class='headding-tree'>Resource Paths</h1><div class='resource-tree-headding'>Picked Path:<input type='text' id='pickedPath' class='picked-path-textbox' /> <input type='button' class='button' onclick=\"putPathToBox('" + textBoxId + "');\" value='OK' /> <input type='button' style='margin-left:10px;' class='button' onclick='showHideCommon('resourceTree');' value='Cancel' /></div>" +
                    resourceTreeData.getResourceTree();
            sendContent(response, displayHTML);

        } catch (RegistryException e) {

            String msg = "Failed to generate the resource tree for the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
        }


    }

    private void fillResourceTree(
            String resourcePath, ResourceTreeData treeData, UserRegistry registry)
            throws RegistryException {


        String[] childPaths = {""};
        Resource resource;
        String resourceName = "";
        String resourceType;
        boolean hasChildren = false;

        resource = registry.get(resourcePath);
        resourceType = resource instanceof Collection ? ResourceData.COLLECTION : ResourceData.RESOURCE;
        if (resourceType.equals(ResourceData.COLLECTION)) {
            childPaths = (String[]) resource.getContent();
            if (childPaths.length > 0) hasChildren = true;
        }
        if (!resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            if (resourcePath != null) {
                String[] parts = resourcePath.split(RegistryConstants.PATH_SEPARATOR);
                resourceName = parts[parts.length - 1];
            }
        } else {
            resourceName = "/";
        }

        resource.discard();

        treeData.incrementTreeIndex();
        treeData.appendToTree("<div class=\"father-object\">");
        if (resourceType != ResourceData.RESOURCE) {
            treeData.appendToTree("<a onclick=\"showHideCommon('y_plus_" + treeData.getResourceTreeIndex() + "');showHideCommon('y_minus_" + treeData.getResourceTreeIndex() + "');showHideCommon('z_" + treeData.getResourceTreeIndex() + "');\">");
            if (hasChildren) {
                treeData.appendToTree("<img src=\"/wso2registry/admin/images/icon-tree-plus.jpg\" id=\"y_plus_" + treeData.getResourceTreeIndex() + "\" style=\"display:none;margin-right:5px;\" />" +
                        "<img src=\"/wso2registry/admin/images/icon-tree-minus.jpg\" id=\"y_minus_" + treeData.getResourceTreeIndex() + "\" style=\"margin-right:5px;\" />");
            } else {
                treeData.appendToTree("<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:20px;height:10px;\" />");
            }
            treeData.appendToTree("<a onclick=\"pickPath('" + resourcePath + "')\">" +
                    "<img src=\"/wso2registry/admin/images/icon-folder-small.gif\" style=\"margin-right:2px;\" />" +
                    resourceName +
                    "</a>");
        } else {
            treeData.appendToTree("<a class=\"plane-resource\" onclick=\"pickPath('" + resourcePath + "')\">" + "<img src=\"/wso2registry/admin/images/spacer.gif\" style=\"width:20px;height:10px;\" />" + resourceName + "</a>");
        }
        treeData.appendToTree("</div>" + "<div class=\"child-objects\" id=\"z_" + treeData.getResourceTreeIndex() + "\">");
        if (resourceType == ResourceData.RESOURCE) {
            treeData.appendToTree("</div>");
            return;
        } else {
            if (!hasChildren) {
                treeData.appendToTree("</div>");
                return;
            } else {

                for (int i = 0; childPaths.length > i; i++) {
                    //Recursive call
                    fillResourceTree(childPaths[i], treeData, registry);
                }
            }
        }
        treeData.appendToTree("</div>");
    }

    private class ResourceTreeData {

        String resourceTree;
        int resourceTreeIndex;

        public ResourceTreeData() {
            resourceTree = "";
            resourceTreeIndex = 1;
        }

        public String getResourceTree() {
            return resourceTree;
        }

        public void setResourceTree(String resourceTree) {
            this.resourceTree = resourceTree;
        }

        public void appendToTree(String treePart) {
            resourceTree += treePart;
        }

        public int getResourceTreeIndex() {
            return resourceTreeIndex;
        }

        public void setResourceTreeIndex(int resourceTreeIndex) {
            this.resourceTreeIndex = resourceTreeIndex;
        }

        public void incrementTreeIndex() {
            resourceTreeIndex++;
        }
    }
}
