/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Resource;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.populators.CommonBeanPopulator;
import org.wso2.registry.web.populators.ContentBeanPopulator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class GivePageProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String wantedPage = request.getParameter("wantedPage");
        String parentPath = request.getParameter("resourcePath");

        ResourcePath parentResourcePath = new ResourcePath(parentPath);
        CommonBeanPopulator.populate(request, parentResourcePath);

        Resource resource;
        try {
            UserRegistry registry = getUserRegistry(request);
            resource = registry.get(parentPath);
        } catch (RegistryException e) {
            String msg = "Failed to get collection resource " + parentPath + ". " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
            return;
        }
        ContentBeanPopulator.populate(request, resource, parentResourcePath);
        resource.discard();

        request.getSession().setAttribute(UIConstants.AJAX_GIVEPAGE_LIST, wantedPage);

        forward(request, response,  UIConstants.AJAX_ENTRY_LIST_JSP);
    }
}
