/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.ActionConstants;
import org.wso2.registry.ResourcePath;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.populators.CommonBeanPopulator;
import org.wso2.registry.web.populators.PermissionsBeanPopulator;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class RoleAuthorizationProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String pathToAuthorize = request.getParameter("pathToAuthorize");
        String roleToAuthorize = request.getParameter("roleToAuthorize");
        String actionToAuthorize = request.getParameter("actionToAuthorize");
        String permissionType = request.getParameter("permissionType");

        UserRealm userRealm ;
        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            userRealm = userRegistry.getUserRealm();
            userRealm.getAccessControlAdmin();
        } catch (Exception e) {
            String msg =
                    "Couldn't get access control admin for changing authorizations. Caused by: " +
                            e.getMessage();
            setSectionErrorMessage(request, msg);
            sendPermissionHTML(request, response, pathToAuthorize);
            return;
        }

        try {
            if (actionToAuthorize.equals("2")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.GET);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.GET);
                }
            }

            if (actionToAuthorize.equals("3")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.PUT);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.PUT);
                }
            }

            if (actionToAuthorize.equals("4")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                }
            }

            if (actionToAuthorize.equals("5")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                }
            }

            String msg = "Role authorization performed successfully.";
            log.info(msg);
            setSuccessMessage(request, msg);

        } catch (UserStoreException e) {
            setSectionErrorMessage(request, e.getMessage());
        }

        sendPermissionHTML(request, response, pathToAuthorize);
    }

    private void sendPermissionHTML(
            HttpServletRequest request, HttpServletResponse response, String resourcePath) {

        try {
            UserRegistry userRegistry = getUserRegistry(request);
            ResourcePath path = new ResourcePath(resourcePath);
            CommonBeanPopulator.populate(request, path);
            PermissionsBeanPopulator.populate(request, userRegistry, path);

        } catch (RegistryException e) {
            String msg = "Failed to get user registry. " + e.getMessage();
            log.error(msg, e);
            setSectionErrorMessage(request, msg);
        }

        forward(request, response, UIConstants.AJAX_PERMISSIONS_JSP);
    }
}
