/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Registry;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.utils.CommonUtil;
import org.wso2.registry.web.actions.UserDetailsAction;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.populators.UserBeanPopulator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

public class SaveFriendlyNameProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String userName = request.getParameter("userName");
        String friendlyName = request.getParameter("friendlyName");

        try {

            UserRegistry userRegistry = getUserRegistry(request);
            UserRealm userRealm = userRegistry.getUserRealm();

            Map userProperties = userRealm.getUserStoreAdmin().getUserProperties(userName);
            userProperties.put("friendlyName", friendlyName);
            userRealm.getUserStoreAdmin().setUserProperties(userName, userProperties);

            UserBeanPopulator.populate(request, userName);
            forward(request, response, UIConstants.AJAX_USER_FRIENDLY_NAME_JSP);

        } catch (Exception e) {

            String msg = "Failed to update the friendly name of the user " +
                    userName + "\nCaused by: " + e.getMessage();
            log.error(msg, e);
            sendErrorContent(response, msg);
        }
    }
}
