/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.web.UIConstants;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class SetDescriptionProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String description = request.getParameter("description");

        try {
            Registry registry = getUserRegistry(request);

            Resource resource = registry.get(resourcePath);
            resource.setDescription(description);
            registry.put(resourcePath, resource);
            resource.discard();

        } catch (RegistryException e) {

            String msg = "Failed to set description for the resource " +
                    resourcePath + ". " + e.getMessage();
            log.error(msg, e);
            setSectionErrorMessage(request, msg);
        }

        request.getSession().setAttribute(UIConstants.AJAX_DESCRIPTION_STRING, description);
        forward(request, response, UIConstants.AJAX_DESCRIPTION_JSP);
    }
}
