/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.web.ConsoleConstants;
import org.wso2.registry.web.UIConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class SignInProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        String userName = request.getParameter("userName");
        String password = request.getParameter("password");

        if (userName.equals(RegistryConstants.SYSTEM_USER)) {
            String msg = "ERROR: Users are not allowed to log in as the system user.";
            log.error(msg);
            sendContent(response, msg);
            return;
        }

        ServletContext context = request.getSession().getServletContext();
        EmbeddedRegistry embeddedRegistry =
                (EmbeddedRegistry) context.getAttribute(RegistryConstants.REGISTRY);

        try {
            UserRealm anonymousRealm = embeddedRegistry.getUserRegistry().getUserRealm();

            if (anonymousRealm.getAuthenticator().authenticate(userName, password)) {

                UserRegistry secureRegistry = embeddedRegistry.getUserRegistry(userName, password);
                request.getSession().setAttribute(ConsoleConstants.USER_REGISTRY, secureRegistry);

                request.getSession().
                        setAttribute(UIConstants.PATH_ATTR, RegistryConstants.ROOT_PATH);

                String msg = "SUCCESS: User authenticated.";
                sendContent(response, msg);
            } else {
                if(userName.contains("<")) userName="";
                String msg = "ERROR: Could not authenticate user " + userName;
                log.error(msg);
                sendContent(response, msg);
            }
        } catch (Exception e) {
            String msg = "ERROR: " + e.getMessage();
            log.error(msg, e);
            sendContent(response, msg);
        }
    }
}
