/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.processors;

import org.wso2.registry.RegistryConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.EmbeddedRegistry;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.ConsoleConstants;
import org.wso2.registry.web.utils.CommonUtil;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class SignOutProcessor extends UIProcessor {

    public void process(HttpServletRequest request, HttpServletResponse response) {

        ServletContext context = request.getSession().getServletContext();
        EmbeddedRegistry embeddedRegistry =
                (EmbeddedRegistry) context.getAttribute(RegistryConstants.REGISTRY);

        try {
            UserRegistry userRegistry = embeddedRegistry.getUserRegistry(
                    RegistryConstants.ANONYMOUS_USER, RegistryConstants.ANONYMOUS_PASSWORD);
            request.getSession().setAttribute(ConsoleConstants.USER_REGISTRY, userRegistry);

        } catch (RegistryException e) {

            String msg = "Failed to sign out the user. " + e.getMessage();
            log.error(msg, e);
            CommonUtil.addErrorMessage(request, msg);
        }

        redirect(response, "/wso2registry/web");
    }
}
