/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.utils;

import org.wso2.registry.announcement.Announcement;
import org.wso2.registry.exceptions.RegistryException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.IOException;

public class AnnouncementsUtil {

    private static Log log = LogFactory.getLog(AnnouncementsUtil.class);

    public static final String ANNOUNCEMENTS_URL =
            "http://wso2.com/announcements/registry/1.0/index.html";

    public static void sendAnnouncement(HttpServletResponse response) throws RegistryException {

        Announcement announcement = new Announcement();
        String annConent = announcement.get(null, ANNOUNCEMENTS_URL);

        if (annConent == null) {
            annConent = "";
        }

        PrintWriter writer = null;
        try {
            writer = response.getWriter();
            writer.println(annConent);
            writer.flush();

        } catch (IOException e) {

            String msg = "Failed to send announcement text. " + e.getMessage();
            log.error(msg, e);
            throw new RegistryException(msg, e);

        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }
}
