package org.wso2.registry.web.utils;

import org.wso2.registry.Aspect;
import org.wso2.registry.Registry;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.jdbc.handlers.RequestContext;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.actions.ResourceDetailsAction;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/** TODO: JavaDoc */
public class LifecycleUtil {
    public static class SimpleLifecycle extends Aspect {
        public static final String STATE_PROP = "SimpleLifecycle.state";
        public static final String INIT = "init";
        public static final String FINAL = "final";

        public static final String ACTION = "invoke";

        public void associate(Resource resource, Registry registry) throws RegistryException {
            resource.setProperty(STATE_PROP, INIT);
        }

        public void dissociate(RequestContext context) {
            context.getResource().removeProperty(STATE_PROP);
        }

        public void invoke(RequestContext context, String action) throws RegistryException {
            if (!ACTION.equals(action)) {
                throw new RegistryException("Wrong action");
            }

            Resource r = context.getResource();
            String state = r.getProperty(STATE_PROP);
            if (state == null) {
                throw new RegistryException("No state property");
            }

            if (!INIT.equals(state)) {
                throw new RegistryException("Invalid state '" + state + "'");
            }

            r.setProperty(STATE_PROP, FINAL);
        }

        static String[] actions = new String[]{ACTION};

        public String[] getAvailableActions(RequestContext context) {
            Resource r = context.getResource();
            String state = r.getProperty(STATE_PROP);
            if (INIT.equals(state)) return actions;
            return null;
        }
    }

    public static void addAspect(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException, RegistryException  {
        try {
            String resourcePath = request.getParameter("resourcePath");
            String aspectName = request.getParameter("aspect");
            Registry registry = CommonUtil.getUserRegistry(request);
            Map<String, String[]> lifecycles = new HashMap<String, String[]>();

            registry.associateAspect(resourcePath, aspectName);
            Resource resource = registry.get(resourcePath);
            List<String> aspectList = resource.getAspects();
            registry.put(resourcePath, resource);

            //We need to update the lifecycle state using property valus
            Properties props=resource.getProperties();

            for (String asp : aspectList) {
                String[] actions = registry.getAspectActions(resourcePath, asp);
                lifecycles.put(asp, actions);
            }
            request.getSession().setAttribute(UIConstants.ACTIVITY_PROPERTY_BEAN, props);
            request.getSession().setAttribute(UIConstants.AJAX_LIFECYCLE_LIST, lifecycles);

            ResourceDetailsAction details = new ResourceDetailsAction();
            details.setPath(resourcePath);
            details.execute(request);

            request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, details);

        } catch (RegistryException e) {
            request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
            throw e;
        }
//        request.getRequestDispatcher(UIConstants.AJAX_LIFECYCLELIST_JSP).forward(request, response);
    }

    public static void removeAspect() {

    }
}
