/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.utils;

import org.wso2.registry.users.AccessControlAdmin;
import org.wso2.registry.users.UserRealm;
import org.wso2.registry.users.UserStoreException;
import org.wso2.registry.users.accesscontrol.AccessControlConstants;
import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.ActionConstants;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.actions.ResourceDetailsAction;
import org.wso2.registry.web.UIConstants;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;

public class PermissionUtil {

    public static void addUserPermission(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String pathToAuthorize = request.getParameter("pathToAuthorize");
        String userToAuthorize = request.getParameter("userToAuthorize");
        String actionToAuthorize = request.getParameter("actionToAuthorize");
        String permissionType = request.getParameter("permissionType");

        UserRealm userRealm ;
        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            userRealm = userRegistry.getUserRealm();
            userRealm.getAccessControlAdmin();

        } catch (Exception e) {
            String msg =
                    "Couldn't get access control admin for changing authorizations. Caused by: " +
                            e.getMessage();
            setErrorMessage(request, msg);
            sendPermissionHTML(request, response, pathToAuthorize);
            return;
        }

        try {
            if (actionToAuthorize.equals("2")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeUser(userToAuthorize, pathToAuthorize, ActionConstants.GET);
                } else {
                    userRealm.getAccessControlAdmin().denyUser(userToAuthorize, pathToAuthorize, ActionConstants.GET);
                }
            }

            if (actionToAuthorize.equals("3")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeUser(userToAuthorize, pathToAuthorize, ActionConstants.PUT);
                } else {
                    userRealm.getAccessControlAdmin().denyUser(userToAuthorize, pathToAuthorize, ActionConstants.PUT);
                }
            }

            if (actionToAuthorize.equals("4")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeUser(userToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                } else {
                    userRealm.getAccessControlAdmin().denyUser(userToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                }
            }

            if (actionToAuthorize.equals("5")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeUser(userToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                } else {
                    userRealm.getAccessControlAdmin().denyUser(userToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                }
            }
        } catch (UserStoreException e) {
            setErrorMessage(request, e.getMessage());
        }

        sendPermissionHTML(request, response, pathToAuthorize);
    }

    public static void applyMassUserPermissions(
            HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String permissionString = request.getParameter("permissionInput");
        String resourcePath = request.getParameter("resourcePath");

        AccessControlAdmin accessControlAdmin ;
        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            UserRealm realm = userRegistry.getUserRealm();
            accessControlAdmin = realm.getAccessControlAdmin();
        } catch (Exception e) {
            String msg =
                    "Couldn't get access control admin for changing authorizations. Caused by: " +
                            e.getMessage();

            setErrorMessage(request, msg);
            sendPermissionHTML(request, response, resourcePath);
            return;
        }

        try {

            String[] userPermissions = permissionString.split("\\|");
            for (int i = 0; i < userPermissions.length; i++) {

                if (userPermissions[i].trim().length() == 0) {
                    continue;
                }

                String[] permissions = userPermissions[i].split(":");
                String permUser = permissions[0];

                if (!(permUser.equals(RegistryConstants.SYSTEM_USER) ||
                        permUser.equals(RegistryConstants.ADMIN_USER))) {

                    accessControlAdmin.
                            clearUserAuthorization(permUser, resourcePath, ActionConstants.GET);
                    accessControlAdmin.
                            clearUserAuthorization(permUser, resourcePath, ActionConstants.PUT);
                    accessControlAdmin.
                            clearUserAuthorization(permUser, resourcePath, ActionConstants.DELETE);
                    accessControlAdmin.clearUserAuthorization(
                            permUser, resourcePath, AccessControlConstants.AUTHORIZE);
                }

                for (int j = 1; j < permissions.length; j++) {
                    String[] permission = permissions[j].split("\\^");

                    String action = permission[0];
                    String checked = permission[1];

                    if (action.equals("ra")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeUser(
                                    permUser, resourcePath, ActionConstants.GET);
                        }
                    } else if (action.equals("rd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyUser(
                                    permUser, resourcePath, ActionConstants.GET);
                        }
                    } else if (action.equals("wa")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeUser(
                                    permUser, resourcePath, ActionConstants.PUT);
                        }
                    } else if (action.equals("wd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyUser(
                                    permUser, resourcePath, ActionConstants.PUT);
                        }
                    } else if (action.equals("da")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeUser(
                                    permUser, resourcePath, ActionConstants.DELETE);
                        }
                    } else if (action.equals("dd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyUser(
                                    permUser, resourcePath, ActionConstants.DELETE);
                        }
                    } else if (action.equals("aa")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeUser(
                                    permUser, resourcePath, AccessControlConstants.AUTHORIZE);
                        }
                    } else if (action.equals("ad")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyUser(
                                    permUser, resourcePath, AccessControlConstants.AUTHORIZE);
                        }
                    }
                }
            }
        } catch (UserStoreException e) {
            String msg = "Couldn't set authorizations. Caused by: " + e.getMessage();
            setErrorMessage(request, msg);
        }

        sendPermissionHTML(request, response, resourcePath);
    }

    public static void addRolePermission(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String pathToAuthorize = request.getParameter("pathToAuthorize");
        String roleToAuthorize = request.getParameter("roleToAuthorize");
        String actionToAuthorize = request.getParameter("actionToAuthorize");
        String permissionType = request.getParameter("permissionType");

        UserRealm userRealm ;
        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            userRealm = userRegistry.getUserRealm();
            userRealm.getAccessControlAdmin();
        } catch (Exception e) {
            String msg =
                    "Couldn't get access control admin for changing authorizations. Caused by: " +
                            e.getMessage();
            setErrorMessage(request, msg);
            sendPermissionHTML(request, response, pathToAuthorize);
            return;
        }

        try {
            if (actionToAuthorize.equals("2")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.GET);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.GET);
                }
            }

            if (actionToAuthorize.equals("3")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.PUT);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.PUT);
                }
            }

            if (actionToAuthorize.equals("4")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, ActionConstants.DELETE);
                }
            }

            if (actionToAuthorize.equals("5")) {

                if (permissionType.equals("1")) {
                    userRealm.getAccessControlAdmin().authorizeRole(roleToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                } else {
                    userRealm.getAccessControlAdmin().denyRole(roleToAuthorize, pathToAuthorize, AccessControlConstants.AUTHORIZE);
                }
            }
        } catch (UserStoreException e) {
            setErrorMessage(request, e.getMessage());
        }

        sendPermissionHTML(request, response, pathToAuthorize);
    }

    public static void applyMassRolePermissions(
            HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String permissionString = request.getParameter("permissionInput");
        String resourcePath = request.getParameter("resourcePath");

        AccessControlAdmin accessControlAdmin ;
        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            UserRealm realm = userRegistry.getUserRealm();
            accessControlAdmin = realm.getAccessControlAdmin();

        } catch (Exception e) {
            String msg = "Couldn't get access control admin for changing authorizations. Caused by: " + e.getMessage();
            setErrorMessage(request, msg);
            sendPermissionHTML(request, response, resourcePath);
            return;
        }

        try {

            String[] rolePermissions = permissionString.split("\\|");
            for (int i = 0; i < rolePermissions.length; i++) {

                if (rolePermissions[i].trim().length() == 0) {
                    continue;
                }

                String[] permissions = rolePermissions[i].split(":");
                String permRole = permissions[0];

                if (!permRole.equals(RegistryConstants.ADMIN_ROLE)) {
                    accessControlAdmin.clearRoleAuthorization(permRole, resourcePath, ActionConstants.GET);
                    accessControlAdmin.clearRoleAuthorization(permRole, resourcePath, ActionConstants.PUT);
                    accessControlAdmin.clearRoleAuthorization(permRole, resourcePath, ActionConstants.DELETE);
                    accessControlAdmin.clearRoleAuthorization(permRole, resourcePath, AccessControlConstants.AUTHORIZE);
                }

                for (int j = 1; j < permissions.length; j++) {
                    String[] permission = permissions[j].split("\\^");

                    String action = permission[0];
                    String checked = permission[1];

                    if (action.equals("ra")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeRole(permRole, resourcePath, ActionConstants.GET);
                        }
                    } else if (action.equals("rd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyRole(permRole, resourcePath, ActionConstants.GET);
                        }
                    } else if (action.equals("wa")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeRole(permRole, resourcePath, ActionConstants.PUT);
                        }
                    } else if (action.equals("wd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyRole(permRole, resourcePath, ActionConstants.PUT);
                        }
                    } else if (action.equals("da")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeRole(permRole, resourcePath, ActionConstants.DELETE);
                        }
                    } else if (action.equals("dd")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyRole(permRole, resourcePath, ActionConstants.DELETE);
                        }
                    } else if (action.equals("aa")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.authorizeRole(permRole, resourcePath, AccessControlConstants.AUTHORIZE);
                        }
                    } else if (action.equals("ad")) {
                        if (checked.equals("true")) {
                            accessControlAdmin.denyRole(permRole, resourcePath, AccessControlConstants.AUTHORIZE);
                        }
                    }
                }
            }
        } catch (UserStoreException e) {
            String msg = "Couldn't set authorizations. Caused by: " + e.getMessage();
            setErrorMessage(request, msg);
        }

        sendPermissionHTML(request, response, resourcePath);
    }

    private static void sendPermissionHTML(
            HttpServletRequest request, HttpServletResponse response, String resourcePath)
            throws ServletException, IOException {

        ResourceDetailsAction resourceDetailsAction = new ResourceDetailsAction();
        resourceDetailsAction.setPath(resourcePath);

        try {
            resourceDetailsAction.execute(request);
        } catch (RegistryException e) {
            request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, e.getMessage());
        }

        request.getSession().setAttribute(UIConstants.RESOURCE_BEAN, resourceDetailsAction);
        request.getRequestDispatcher(UIConstants.AJAX_PERMISSIONS_JSP).forward(request, response);
    }

    private static void setErrorMessage(HttpServletRequest request, String msg) {
        request.getSession().setAttribute(UIConstants.ERROR_MESSAGE, msg);
    }
}
