/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.utils;

import org.wso2.registry.exceptions.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.session.UserRegistry;
import org.wso2.registry.web.UIConstants;
import org.wso2.registry.web.populators.PropertiesBeanPopulator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.util.Properties;
import java.io.IOException;

public class PropertiesUtil {

    public static void setProperty(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String propertyName = request.getParameter("propertyName");
        String propertyValue = request.getParameter("propertyValue");

        try {
            setProperty(resourcePath, propertyName, propertyValue, request);

        } catch (RegistryException e) {
            e.printStackTrace();
        }

        try {
            request.getSession().setAttribute("resourcePath", resourcePath);
            request.getRequestDispatcher(UIConstants.AJAX_PROPERTIES_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void updateProperty(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String oldPropertyName = request.getParameter("oldPropertyName");
        String propertyName = request.getParameter("propertyName");
        String propertyValue = request.getParameter("propertyValue");

        try {
            UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
            Resource resource = userRegistry.get(resourcePath);

            if (oldPropertyName.equals(propertyName)) {
                resource.setProperty(propertyName, propertyValue);
            } else {
                resource.setProperty(propertyName, propertyValue);
                resource.removeProperty(oldPropertyName);
                //Properties properties = resource.getProperties();
                //properties.remove(oldPropertyName);
                //properties.setProperty(propertyName, propertyValue);
                //resource.setProperties(properties);
            }

            userRegistry.put(resource.getPath(), resource);

        } catch (RegistryException e) {
            e.printStackTrace();
        }

        try {
            request.getSession().setAttribute("resourcePath", resourcePath);
            request.getRequestDispatcher(UIConstants.AJAX_PROPERTIES_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void removeProperty(HttpServletRequest request, HttpServletResponse response) {

        String resourcePath = request.getParameter("resourcePath");
        String propertyName = request.getParameter("propertyName");

        try {
            removeProperty(resourcePath, propertyName, request);

        } catch (RegistryException e) {
            e.printStackTrace();
        }

        try {
            request.getSession().setAttribute("resourcePath", resourcePath);
            request.getRequestDispatcher(UIConstants.AJAX_PROPERTIES_JSP).forward(request, response);
        } catch (ServletException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Properties getResourceProperties(String resourcePath, HttpServletRequest request)
            throws RegistryException {

        UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
        Resource resource = userRegistry.get(resourcePath);

        return resource.getProperties();
    }

    public static void setProperty(
            String resourcePath,
            String propertyName,
            String propertyValue,
            HttpServletRequest request) throws RegistryException {

        UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
        Resource resource = userRegistry.get(resourcePath);
        resource.addProperty(propertyName, propertyValue);
        userRegistry.put(resource.getPath(), resource);
    }

    public static void removeProperty(String resourcePath, String propertyName, HttpServletRequest request) throws RegistryException {

        UserRegistry userRegistry = CommonUtil.getUserRegistry(request);
        Resource resource = userRegistry.get(resourcePath);
        resource.removeProperty(propertyName);
        userRegistry.put(resource.getPath(), resource);
    }
}
